/****************************************************************************
 *                           I V I - P O W E R                               
 *---------------------------------------------------------------------------
 *    Copyright (c) National Instruments 1998.  All Rights Reserved.        
 *---------------------------------------------------------------------------
 *                                                                          
 * Title:       ivipower.h                                                    
 * Purpose:     IviPower Class declarations for the Fundamental and Extended  
 *              IviPower Capabilities.                                        
 *                                                                          
 ****************************************************************************/

#ifndef IVIPOWER_HEADER
#define IVIPOWER_HEADER

#include <ivi.h>

#ifdef _CVI_
#pragma EnableLibraryRuntimeChecking
#endif

#if defined(__cplusplus) || defined(__cplusplus__)
extern "C" {
#endif

/****************************************************************************
 *----------------- Instrument Driver Revision Information -----------------*
 ****************************************************************************/

#define IVIPOWER_MAJOR_VERSION                  1
#define IVIPOWER_MINOR_VERSION                  0

/****************************************************************************
 *------------------------------ Useful Macros -----------------------------*
 ****************************************************************************/

    /* Defined values for specifying the type of a waveform or transient. */
#define IVIPOWER_VAL_VOLTAGE                    1
#define IVIPOWER_VAL_CURRENT                    2

    /* Defined values for maxTime parameter to the Read and Fetch functions */
#define IVIPOWER_VAL_MAX_TIME_INFINITE          IVI_VAL_MAX_TIME_INFINITE
#define IVIPOWER_VAL_MAX_TIME_IMMEDIATE         IVI_VAL_MAX_TIME_IMMEDIATE

/****************************************************************************
 *------------------ IviPower Class Attribute Defines ----------------------*
 ****************************************************************************/

    /*- IVI Inherent Attributes --------------------------------------------*/

        /* User Options */
#define IVIPOWER_ATTR_CACHE                     IVI_ATTR_CACHE                         /* ViBoolean */
#define IVIPOWER_ATTR_RANGE_CHECK               IVI_ATTR_RANGE_CHECK                   /* ViBoolean */
#define IVIPOWER_ATTR_QUERY_INSTR_STATUS        IVI_ATTR_QUERY_INSTR_STATUS            /* ViBoolean */
#define IVIPOWER_ATTR_RECORD_COERCIONS          IVI_ATTR_RECORD_COERCIONS              /* ViBoolean */
#define IVIPOWER_ATTR_SIMULATE                  IVI_ATTR_SIMULATE                      /* ViBoolean */
#define IVIPOWER_ATTR_USE_SPECIFIC_SIMULATION   IVI_ATTR_USE_SPECIFIC_SIMULATION       /* ViBoolean */
#define IVIPOWER_ATTR_INTERCHANGE_CHECK         IVI_ATTR_INTERCHANGE_CHECK             /* ViBoolean */
#define IVIPOWER_ATTR_SPY                       IVI_ATTR_SPY                           /* ViBoolean */

        /* Instrument Capabilities */
#define IVIPOWER_ATTR_NUM_CHANNELS              IVI_ATTR_NUM_CHANNELS                  /* ViInt32,  read-only */
#define IVIPOWER_ATTR_GROUP_CAPABILITIES        IVI_ATTR_GROUP_CAPABILITIES            /* ViString, read-only */
#define IVIPOWER_ATTR_FUNCTION_CAPABILITIES     IVI_ATTR_FUNCTION_CAPABILITIES         /* ViString, read-only */
#define IVIPOWER_ATTR_ATTRIBUTE_CAPABILITIES    IVI_ATTR_ATTRIBUTE_CAPABILITIES        /* ViString, read-only */

        /* Driver Information */    
#define IVIPOWER_ATTR_CLASS_PREFIX              IVI_ATTR_CLASS_PREFIX                  /* ViString, read-only */
#define IVIPOWER_ATTR_SPECIFIC_PREFIX           IVI_ATTR_SPECIFIC_PREFIX               /* ViString, read-only */
#define IVIPOWER_ATTR_MODULE_PATHNAME           IVI_ATTR_MODULE_PATHNAME               /* ViString, read-only */
#define IVIPOWER_ATTR_RESOURCE_DESCRIPTOR       IVI_ATTR_RESOURCE_DESCRIPTOR           /* ViString, read-only */
#define IVIPOWER_ATTR_LOGICAL_NAME              IVI_ATTR_LOGICAL_NAME                  /* ViString, read-only */

        /* Version Info */
#define IVIPOWER_ATTR_CLASS_MAJOR_VERSION       IVI_ATTR_CLASS_MAJOR_VERSION           /* ViInt32,  read-only */
#define IVIPOWER_ATTR_CLASS_MINOR_VERSION       IVI_ATTR_CLASS_MINOR_VERSION           /* ViInt32,  read-only */
#define IVIPOWER_ATTR_CLASS_REVISION            IVI_ATTR_CLASS_REVISION                /* ViString, read-only */

#define IVIPOWER_ATTR_DRIVER_MAJOR_VERSION      IVI_ATTR_DRIVER_MAJOR_VERSION          /* ViInt32,  read-only */
#define IVIPOWER_ATTR_DRIVER_MINOR_VERSION      IVI_ATTR_DRIVER_MINOR_VERSION          /* ViInt32,  read-only */
#define IVIPOWER_ATTR_DRIVER_REVISION           IVI_ATTR_DRIVER_REVISION               /* ViString, read-only */

#define IVIPOWER_ATTR_ENGINE_MAJOR_VERSION      IVI_ATTR_ENGINE_MAJOR_VERSION          /* ViInt32,  read-only */
#define IVIPOWER_ATTR_ENGINE_MINOR_VERSION      IVI_ATTR_ENGINE_MINOR_VERSION          /* ViInt32,  read-only */
#define IVIPOWER_ATTR_ENGINE_REVISION           IVI_ATTR_ENGINE_REVISION               /* ViString, read-only */

        /* Error Info */
#define IVIPOWER_ATTR_PRIMARY_ERROR             IVI_ATTR_PRIMARY_ERROR                 /* ViInt32  */
#define IVIPOWER_ATTR_SECONDARY_ERROR           IVI_ATTR_SECONDARY_ERROR               /* ViInt32  */
#define IVIPOWER_ATTR_ERROR_ELABORATION         IVI_ATTR_ERROR_ELABORATION             /* ViString */

        /* Driver Setup information */
#define IVIPOWER_ATTR_DRIVER_SETUP             IVI_ATTR_DRIVER_SETUP                  /* ViString */

    /*- IviPower Fundamental Attributes ------------------------------------*/

#define IVIPOWER_ATTR_WAVEFORM                  (IVI_CLASS_PUBLIC_ATTR_BASE + 1L)    /* ViInt32,   multi-channel                     */
#define IVIPOWER_ATTR_VOLTAGE_LIMIT             (IVI_CLASS_PUBLIC_ATTR_BASE + 2L)    /* ViReal64,  multi-channel, volts              */
#define IVIPOWER_ATTR_CURRENT_LIMIT             (IVI_CLASS_PUBLIC_ATTR_BASE + 3L)    /* ViReal64,  multi-channel, amperes            */
#define IVIPOWER_ATTR_OVP_ENABLED               (IVI_CLASS_PUBLIC_ATTR_BASE + 4L)    /* ViBoolean, multi-channel                     */
#define IVIPOWER_ATTR_OCP_ENABLED               (IVI_CLASS_PUBLIC_ATTR_BASE + 5L)    /* ViBoolean, multi-channel                     */
#define IVIPOWER_ATTR_OUTPUT_CONNECTED          (IVI_CLASS_PUBLIC_ATTR_BASE + 6L)    /* ViBoolean, multi-channel                     */
#define IVIPOWER_ATTR_PEAK_VOLTAGE              (IVI_CLASS_PUBLIC_ATTR_BASE + 8L)    /* ViReal64,  multi-channel, read-only, volts   */
#define IVIPOWER_ATTR_PEAK_CURRENT              (IVI_CLASS_PUBLIC_ATTR_BASE + 9L)    /* ViReal64,  multi-channel, read-only, amperes */

    /*- IviPowerDCV Voltage Source Group -*/
#define IVIPOWER_ATTR_DC_VOLTAGE                (IVI_CLASS_PUBLIC_ATTR_BASE + 101L)  /* ViReal64, multi-channel, volts */

    /*- IviPowerDCC Current Source Group -*/
#define IVIPOWER_ATTR_DC_CURRENT                (IVI_CLASS_PUBLIC_ATTR_BASE + 201L)  /* ViReal64, multi-channel, amperes */

    /*- IviPowerAC Fundamental Group -*/
#define IVIPOWER_ATTR_FREQUENCY                 (IVI_CLASS_PUBLIC_ATTR_BASE + 301L)  /* ViReal64, multi-channel, hertz   */
#define IVIPOWER_ATTR_START_PHASE               (IVI_CLASS_PUBLIC_ATTR_BASE + 302L)  /* ViReal64, multi-channel, degrees */

    /*- IviPowerACV Voltage Source Group -*/
#define IVIPOWER_ATTR_ACV_AMPLITUDE             (IVI_CLASS_PUBLIC_ATTR_BASE + 401L)  /* ViReal64, multi-channel, volts RMS */
#define IVIPOWER_ATTR_ACV_DC_OFFSET             (IVI_CLASS_PUBLIC_ATTR_BASE + 402L)  /* ViReal64, multi-channel, volts     */

    /*- IviPowerACC Current Source Group -*/
#define IVIPOWER_ATTR_ACC_AMPLITUDE             (IVI_CLASS_PUBLIC_ATTR_BASE + 501L)  /* ViReal64, multi-channel, amperes RMS */
#define IVIPOWER_ATTR_ACC_DC_OFFSET             (IVI_CLASS_PUBLIC_ATTR_BASE + 502L)  /* ViReal64, multi-channel, amperes     */

    /*- IviPower Extended Attributes ---------------------------------------*/

    /*- IviPowerUserWaveform Extension Group -*/
#define IVIPOWER_ATTR_WAVEFORM_MIN_SAMPLES      (IVI_CLASS_PUBLIC_ATTR_BASE + 601L)  /* ViInt32, multi-channel, read-only */
#define IVIPOWER_ATTR_WAVEFORM_MAX_SAMPLES      (IVI_CLASS_PUBLIC_ATTR_BASE + 602L)  /* ViInt32, multi-channel, read-only */

    /*- IviPowerTransient Extension Group -*/
#define IVIPOWER_ATTR_TRANSIENT                 (IVI_CLASS_PUBLIC_ATTR_BASE + 701L)  /* ViInt32,  multi-channel                     */
#define IVIPOWER_ATTR_TRANSIENT_COUNT           (IVI_CLASS_PUBLIC_ATTR_BASE + 702L)  /* ViInt32,  multi-channel                     */
#define IVIPOWER_ATTR_TRANSIENT_REPEAT_DELAY    (IVI_CLASS_PUBLIC_ATTR_BASE + 703L)  /* ViReal64, multi-channel, seconds            */
#define IVIPOWER_ATTR_TRANSIENT_MIN_WAVEFORMS   (IVI_CLASS_PUBLIC_ATTR_BASE + 704L)  /* ViInt32,  multi-channel, read-only          */
#define IVIPOWER_ATTR_TRANSIENT_MAX_WAVEFORMS   (IVI_CLASS_PUBLIC_ATTR_BASE + 705L)  /* ViInt32,  multi-channel, read-only          */
#define IVIPOWER_ATTR_TRANSIENT_MIN_DWELL_TIME  (IVI_CLASS_PUBLIC_ATTR_BASE + 706L)  /* ViReal64, multi-channel, read-only, seconds */
#define IVIPOWER_ATTR_TRANSIENT_MAX_DWELL_TIME  (IVI_CLASS_PUBLIC_ATTR_BASE + 707L)  /* ViReal64, multi-channel, read-only, seconds */

    /*- IviPowerTrigger Extension Group -*/
#define IVIPOWER_ATTR_TRIGGER_SOURCE            (IVI_CLASS_PUBLIC_ATTR_BASE + 801L)  /* ViInt32           */
#define IVIPOWER_ATTR_TRIGGER_DELAY             (IVI_CLASS_PUBLIC_ATTR_BASE + 802L)  /* ViReal64, seconds */
#define IVIPOWER_ATTR_TRIGGER_SLOPE             (IVI_CLASS_PUBLIC_ATTR_BASE + 803L)  /* ViInt32           */

    /*- IviPowerMeasure Extension Group -*/
#define IVIPOWER_ATTR_MEASURE_FUNCTION          (IVI_CLASS_PUBLIC_ATTR_BASE + 901L)  /* ViInt32, multi-channel */

    /*- IviPowerMeasureWithTrigger Extension Group -*/
#define IVIPOWER_ATTR_MEASURE_TRIGGER_SOURCE    (IVI_CLASS_PUBLIC_ATTR_BASE + 1001L) /* ViInt32 */

    /*- IviPowerMeasureMultiPoint Extension Group -*/
#define IVIPOWER_ATTR_MEASURE_SAMPLE_COUNT      (IVI_CLASS_PUBLIC_ATTR_BASE + 1101L) /* ViInt32,  multi-channel          */
#define IVIPOWER_ATTR_MEASURE_SAMPLE_OFFSET     (IVI_CLASS_PUBLIC_ATTR_BASE + 1102L) /* ViReal64, multi-channel, percent */
#define IVIPOWER_ATTR_MEASURE_SAMPLE_INTERVAL   (IVI_CLASS_PUBLIC_ATTR_BASE + 1103L) /* ViReal64, multi-channel, seconds */

    /*- IviPower Miscellaneous Extensions -*/
#define IVIPOWER_ATTR_RESISTIVE_IMPEDANCE       (IVI_CLASS_PUBLIC_ATTR_BASE + 2001L) /* ViReal64, multi-channel, ohms   */
#define IVIPOWER_ATTR_INDUCTIVE_IMPEDANCE       (IVI_CLASS_PUBLIC_ATTR_BASE + 2002L) /* ViReal64, multi-channel, henrys */
#define IVIPOWER_ATTR_CAPACITIVE_IMPEDANCE      (IVI_CLASS_PUBLIC_ATTR_BASE + 2003L) /* ViReal64, multi-channel, farads */

/****************************************************************************
 *----------------- IviPower Class Attribute Value Defines -----------------*
 ****************************************************************************/

    /*- Defined values for attribute IVIPOWER_ATTR_WAVEFORM ----------------*/

#define IVIPOWER_VAL_WAVEFORM_BASE                      0
#define IVIPOWER_VAL_WAVEFORM_DC                        (IVIPOWER_VAL_WAVEFORM_BASE + 1L)
#define IVIPOWER_VAL_WAVEFORM_SINE                      (IVIPOWER_VAL_WAVEFORM_BASE + 2L)
#define IVIPOWER_VAL_WAVEFORM_SQUARE                    (IVIPOWER_VAL_WAVEFORM_BASE + 3L)
#define IVIPOWER_VAL_WAVEFORM_CLASS_EXT_BASE            (IVIPOWER_VAL_WAVEFORM_BASE + 100L)
#define IVIPOWER_VAL_WAVEFORM_SPECIFIC_EXT_BASE         (IVIPOWER_VAL_WAVEFORM_BASE + 10000L)
#define IVIPOWER_VAL_WAVEFORM_FIRST_HANDLE              (IVIPOWER_VAL_WAVEFORM_CLASS_EXT_BASE + 1000L)
#define IVIPOWER_VAL_WAVEFORM_LAST_HANDLE               (IVIPOWER_VAL_WAVEFORM_SPECIFIC_EXT_BASE - 1L)

    /*- Defined values for attribute IVIPOWER_ATTR_TRANSIENT ---------------*/

#define IVIPOWER_VAL_TRANSIENT_BASE                     100000
#define IVIPOWER_VAL_TRANSIENT_NONE                     (IVIPOWER_VAL_TRANSIENT_BASE + 1L)
#define IVIPOWER_VAL_TRANSIENT_CLASS_EXT_BASE           (IVIPOWER_VAL_TRANSIENT_BASE + 100L)
#define IVIPOWER_VAL_TRANSIENT_SPECIFIC_EXT_BASE        (IVIPOWER_VAL_TRANSIENT_BASE + 10000L)
#define IVIPOWER_VAL_TRANSIENT_FIRST_HANDLE             (IVIPOWER_VAL_TRANSIENT_CLASS_EXT_BASE + 1000L)
#define IVIPOWER_VAL_TRANSIENT_LAST_HANDLE              (IVIPOWER_VAL_TRANSIENT_SPECIFIC_EXT_BASE - 1L)

    /*- Defined values for attribute IVIPOWER_ATTR_TRANSIENT_COUNT ---------*/

#define IVIPOWER_VAL_TRANSIENT_COUNT_INFINITE           -1

    /*- Defined values for attribute IVIPOWER_ATTR_TRIGGER_SOURCE ----------*/

#define IVIPOWER_VAL_IMMEDIATE                          1
#define IVIPOWER_VAL_EXTERNAL                           2
#define IVIPOWER_VAL_SW_TRIG_FUNC                       3
#define IVIPOWER_VAL_TRIGGER_SOURCE_CLASS_EXT_BASE      100
#define IVIPOWER_VAL_TRIGGER_SOURCE_SPECIFIC_EXT_BASE   1000

    /*- Defined extended values for attribute IVIPOWER_ATTR_TRIGGER_SOURCE -*/
    
#define IVIPOWER_VAL_GPIB_GET                           (IVIPOWER_VAL_TRIGGER_SOURCE_CLASS_EXT_BASE + 1)
#define IVIPOWER_VAL_TTL0                               (IVIPOWER_VAL_TRIGGER_SOURCE_CLASS_EXT_BASE + 11)
#define IVIPOWER_VAL_TTL1                               (IVIPOWER_VAL_TRIGGER_SOURCE_CLASS_EXT_BASE + 12)
#define IVIPOWER_VAL_TTL2                               (IVIPOWER_VAL_TRIGGER_SOURCE_CLASS_EXT_BASE + 13)
#define IVIPOWER_VAL_TTL3                               (IVIPOWER_VAL_TRIGGER_SOURCE_CLASS_EXT_BASE + 14)
#define IVIPOWER_VAL_TTL4                               (IVIPOWER_VAL_TRIGGER_SOURCE_CLASS_EXT_BASE + 15)
#define IVIPOWER_VAL_TTL5                               (IVIPOWER_VAL_TRIGGER_SOURCE_CLASS_EXT_BASE + 16)
#define IVIPOWER_VAL_TTL6                               (IVIPOWER_VAL_TRIGGER_SOURCE_CLASS_EXT_BASE + 17)
#define IVIPOWER_VAL_TTL7                               (IVIPOWER_VAL_TRIGGER_SOURCE_CLASS_EXT_BASE + 18)
#define IVIPOWER_VAL_ECL0                               (IVIPOWER_VAL_TRIGGER_SOURCE_CLASS_EXT_BASE + 19)
#define IVIPOWER_VAL_ECL1                               (IVIPOWER_VAL_TRIGGER_SOURCE_CLASS_EXT_BASE + 20)
#define IVIPOWER_VAL_PXI_STAR                           (IVIPOWER_VAL_TRIGGER_SOURCE_CLASS_EXT_BASE + 31)
#define IVIPOWER_VAL_RTSI_0                             (IVIPOWER_VAL_TRIGGER_SOURCE_CLASS_EXT_BASE + 41)
#define IVIPOWER_VAL_RTSI_1                             (IVIPOWER_VAL_TRIGGER_SOURCE_CLASS_EXT_BASE + 42)
#define IVIPOWER_VAL_RTSI_2                             (IVIPOWER_VAL_TRIGGER_SOURCE_CLASS_EXT_BASE + 43)
#define IVIPOWER_VAL_RTSI_3                             (IVIPOWER_VAL_TRIGGER_SOURCE_CLASS_EXT_BASE + 44)
#define IVIPOWER_VAL_RTSI_4                             (IVIPOWER_VAL_TRIGGER_SOURCE_CLASS_EXT_BASE + 45)
#define IVIPOWER_VAL_RTSI_5                             (IVIPOWER_VAL_TRIGGER_SOURCE_CLASS_EXT_BASE + 46)
#define IVIPOWER_VAL_RTSI_6                             (IVIPOWER_VAL_TRIGGER_SOURCE_CLASS_EXT_BASE + 47)
    
    /*- Defined values for attribute IVIPOWER_ATTR_TRIGGER_SLOPE -----------*/
    
#define IVIPOWER_VAL_POSITIVE                           1
#define IVIPOWER_VAL_NEGATIVE                           2
    
    /*- Defined values for attribute IVIPOWER_ATTR_MEASURE_FUNCTION --------*/

#define IVIPOWER_VAL_DC_VOLTS                           1
#define IVIPOWER_VAL_AC_VOLTS                           2
#define IVIPOWER_VAL_DC_CURRENT                         3
#define IVIPOWER_VAL_AC_CURRENT                         4
#define IVIPOWER_VAL_AC_PLUS_DC_VOLTS                   5
#define IVIPOWER_VAL_AC_PLUS_DC_CURRENT                 6
#define IVIPOWER_VAL_DC_POWER                           7
#define IVIPOWER_VAL_AC_POWER                           8
#define IVIPOWER_VAL_FREQUENCY                          9
#define IVIPOWER_VAL_MEASURE_FUNCTION_CLASS_EXT_BASE    100
#define IVIPOWER_VAL_MEASURE_FUNCTION_SPECIFIC_EXT_BASE 1000

    /*- Defined values for attribute IVIPOWER_ATTR_MEASURE_TRIGGER_SOURCE --*/

/* #define IVIPOWER_VAL_IMMEDIATE                        DEFINED ABOVE */
/* #define IVIPOWER_VAL_EXTERNAL                         DEFINED ABOVE */
/* #define IVIPOWER_VAL_SW_TRIG_FUNC                     DEFINED ABOVE */
/* #define IVIPOWER_VAL_GPIB_GET                         DEFINED ABOVE */
/* #define IVIPOWER_VAL_TTL0                             DEFINED ABOVE */
/* #define IVIPOWER_VAL_TTL1                             DEFINED ABOVE */
/* #define IVIPOWER_VAL_TTL2                             DEFINED ABOVE */
/* #define IVIPOWER_VAL_TTL3                             DEFINED ABOVE */
/* #define IVIPOWER_VAL_TTL4                             DEFINED ABOVE */
/* #define IVIPOWER_VAL_TTL5                             DEFINED ABOVE */
/* #define IVIPOWER_VAL_TTL6                             DEFINED ABOVE */
/* #define IVIPOWER_VAL_TTL7                             DEFINED ABOVE */
/* #define IVIPOWER_VAL_ECL0                             DEFINED ABOVE */
/* #define IVIPOWER_VAL_ECL1                             DEFINED ABOVE */
/* #define IVIPOWER_VAL_PXI_STAR                         DEFINED ABOVE */
/* #define IVIPOWER_VAL_RTSI_0                           DEFINED ABOVE */
/* #define IVIPOWER_VAL_RTSI_1                           DEFINED ABOVE */
/* #define IVIPOWER_VAL_RTSI_2                           DEFINED ABOVE */
/* #define IVIPOWER_VAL_RTSI_3                           DEFINED ABOVE */
/* #define IVIPOWER_VAL_RTSI_4                           DEFINED ABOVE */
/* #define IVIPOWER_VAL_RTSI_5                           DEFINED ABOVE */
/* #define IVIPOWER_VAL_RTSI_6                           DEFINED ABOVE */

/****************************************************************************
 *--------- IviPower Class Instrument Driver Function Declarations ---------*
 ****************************************************************************/

    /*- Required VXIplug&play Functions ------------------------------------*/
    
ViStatus _VI_FUNC IviPower_init           (ViChar logicalName[], ViBoolean idQuery, ViBoolean resetDevice, ViSession *vi);
ViStatus _VI_FUNC IviPower_close          (ViSession vi);
ViStatus _VI_FUNC IviPower_reset          (ViSession vi);
ViStatus _VI_FUNC IviPower_self_test      (ViSession vi, ViInt16 *selfTestResult, ViChar selfTestMessage[]);
ViStatus _VI_FUNC IviPower_error_query    (ViSession vi, ViInt32 *errorCode, ViChar errorMessage[]);
ViStatus _VI_FUNC IviPower_error_message  (ViSession vi, ViStatus statusCode, ViChar message[]);
ViStatus _VI_FUNC IviPower_revision_query (ViSession vi, ViChar driverRev[], ViChar instrRev[]);

    /*- Required IVI Functions ---------------------------------------------*/

ViStatus _VI_FUNC IviPower_IviInit  (ViChar logicalName[], ViBoolean idQuery, ViBoolean resetDevice, ViSession vi);
ViStatus _VI_FUNC IviPower_IviClose (ViSession vi);

    /*- Set, Get, and Check Attribute Functions -*/
ViStatus _VI_FUNC IviPower_GetAttributeViInt32     (ViSession vi, ViConstString channelName, ViAttr attributeId, ViInt32 *value);
ViStatus _VI_FUNC IviPower_SetAttributeViInt32     (ViSession vi, ViConstString channelName, ViAttr attributeId, ViInt32 value);
ViStatus _VI_FUNC IviPower_CheckAttributeViInt32   (ViSession vi, ViConstString channelName, ViAttr attributeId, ViInt32 value);

ViStatus _VI_FUNC IviPower_GetAttributeViReal64    (ViSession vi, ViConstString channelName, ViAttr attributeId, ViReal64 *value);
ViStatus _VI_FUNC IviPower_SetAttributeViReal64    (ViSession vi, ViConstString channelName, ViAttr attributeId, ViReal64 value);
ViStatus _VI_FUNC IviPower_CheckAttributeViReal64  (ViSession vi, ViConstString channelName, ViAttr attributeId, ViReal64 value);

ViStatus _VI_FUNC IviPower_GetAttributeViString    (ViSession vi, ViConstString channelName, ViAttr attributeId, ViInt32 bufferSize, ViChar value[]);
ViStatus _VI_FUNC IviPower_SetAttributeViString    (ViSession vi, ViConstString channelName, ViAttr attributeId, ViConstString value);
ViStatus _VI_FUNC IviPower_CheckAttributeViString  (ViSession vi, ViConstString channelName, ViAttr attributeId, ViConstString value); 

ViStatus _VI_FUNC IviPower_GetAttributeViBoolean   (ViSession vi, ViConstString channelName, ViAttr attributeId, ViBoolean *value);
ViStatus _VI_FUNC IviPower_SetAttributeViBoolean   (ViSession vi, ViConstString channelName, ViAttr attributeId, ViBoolean value);
ViStatus _VI_FUNC IviPower_CheckAttributeViBoolean (ViSession vi, ViConstString channelName, ViAttr attributeId, ViBoolean value);

ViStatus _VI_FUNC IviPower_GetAttributeViSession   (ViSession vi, ViConstString channelName, ViAttr attributeId, ViSession *value);
ViStatus _VI_FUNC IviPower_SetAttributeViSession   (ViSession vi, ViConstString channelName, ViAttr attributeId, ViSession value);
ViStatus _VI_FUNC IviPower_CheckAttributeViSession (ViSession vi, ViConstString channelName, ViAttr attributeId, ViSession value);

    /*- Lock and Unlock Functions -*/
ViStatus _VI_FUNC IviPower_LockSession   (ViSession vi, ViBoolean *callerHasLock);
ViStatus _VI_FUNC IviPower_UnlockSession (ViSession vi, ViBoolean *callerHasLock);

    /*- Error Information Functions -*/
ViStatus _VI_FUNC IviPower_GetErrorInfo   (ViSession vi, ViStatus *primaryError, ViStatus *secondaryError, ViChar errorElaboration[IVI_MAX_MESSAGE_BUF_SIZE]);
ViStatus _VI_FUNC IviPower_ClearErrorInfo (ViSession vi);

    /*- Interchangeability Checking Functions -*/
ViStatus _VI_FUNC IviPower_GetNextInterchangeWarning (ViSession vi, ViInt32 bufferSize, ViChar warning[]);
ViStatus _VI_FUNC IviPower_GetNextCoercionRecord     (ViSession vi, ViInt32 bufferSize, ViChar record[]);

    /*- IviPower Class Fundamental Functions -------------------------------*/
    
ViStatus _VI_FUNC IviPower_ConnectOutput         (ViSession vi, ViConstString channel);

ViStatus _VI_FUNC IviPower_DisconnectOutput      (ViSession vi, ViConstString channel);

ViStatus _VI_FUNC IviPower_ConfigureLimits       (ViSession vi, ViConstString channel,
                                                  ViReal64 voltageLimit, ViReal64 currentLimit,
                                                  ViBoolean ovpEnabled, ViBoolean ocpEnabled);

ViStatus _VI_FUNC IviPower_Initiate              (ViSession vi);

ViStatus _VI_FUNC IviPower_Abort                 (ViSession vi);

ViStatus _VI_FUNC IviPower_QueryOutputProtection (ViSession vi, ViConstString channel,
                                                  ViBoolean* ovpTripped, ViBoolean* ocpTripped);

ViStatus _VI_FUNC IviPower_ResetOutputProtection (ViSession vi, ViConstString channel);

    /*- IviPowerDCV Functions ----------------------------------------------*/

ViStatus _VI_FUNC IviPower_ConfigureDCV (ViSession vi, ViConstString channel,
                                         ViReal64 dcVoltage);
                                                 
    /*- IviPowerDCC Functions ----------------------------------------------*/
    
ViStatus _VI_FUNC IviPower_ConfigureDCC (ViSession vi, ViConstString channel,
                                         ViReal64 dcCurrent);
                                                 
    /*- IviPowerACV Functions ----------------------------------------------*/
    
ViStatus _VI_FUNC IviPower_ConfigureACV (ViSession vi, ViConstString channel,
                                         ViInt32 waveform, ViReal64 frequency,
                                         ViReal64 amplitude, ViReal64 dcOffset,
                                         ViReal64 startPhase);

    /*- IviPowerACC Functions ----------------------------------------------*/
    
ViStatus _VI_FUNC IviPower_ConfigureACC (ViSession vi, ViConstString channel,
                                         ViInt32 waveform, ViReal64 frequency,
                                         ViReal64 amplitude, ViReal64 dcOffset,
                                         ViReal64 startPhase);

    /*- IviPowerUserWaveform Functions -------------------------------------*/
    
ViStatus _VI_FUNC IviPower_CreateWaveform    (ViSession vi, ViInt32 numberOfSamples,
                                              ViReal64 samples[], ViInt32* waveform);

ViStatus _VI_FUNC IviPower_ClearWaveform     (ViSession vi, ViInt32 waveform);

ViStatus _VI_FUNC IviPower_ClearAllWaveforms (ViSession vi);

    /*- IviPowerTransient Functions ----------------------------------------*/
    
ViStatus _VI_FUNC IviPower_CreateTransient    (ViSession vi, ViInt32 transientType,
                                               ViInt32 numberOfWaveforms,
                                               ViInt32 waveforms[], ViReal64 frequencies[],
                                               ViReal64 amplitudes[], ViReal64 dcOffsets[],
                                               ViReal64 startPhases[], ViReal64 dwellTimes[],
                                               ViInt32* transient);
                                                     
ViStatus _VI_FUNC IviPower_ClearTransient     (ViSession vi, ViInt32 transient);
                                                     
ViStatus _VI_FUNC IviPower_ClearAllTransients (ViSession vi);
                                                     
ViStatus _VI_FUNC IviPower_ConfigureTransient (ViSession vi, ViConstString channel,
                                               ViInt32 transient,
                                               ViInt32 count, ViReal64 repeatDelay);
                                                     
    /*- IviPowerTrigger Functions ------------------------------------------*/
    
ViStatus _VI_FUNC IviPower_SendSWTrigger    (ViSession vi);

ViStatus _VI_FUNC IviPower_ConfigureTrigger (ViSession vi, ViInt32 triggerSource,
                                             ViReal64 triggerDelay);

    /*- IviPowerMeasure Functions ------------------------------------------*/
    
ViStatus _VI_FUNC IviPower_InitiateMeasurement  (ViSession vi);

ViStatus _VI_FUNC IviPower_ConfigureMeasurement (ViSession vi, ViConstString channel,
                                                 ViInt32 measureFunction);

ViStatus _VI_FUNC IviPower_FetchMeasurement     (ViSession vi, ViConstString channel,
                                                 ViInt32 maxTime, ViReal64 *reading);

ViStatus _VI_FUNC IviPower_ReadMeasurement      (ViSession vi, ViConstString channel,
                                                 ViInt32 maxTime, ViReal64 *reading);

    /*- IviPowerMeasureWithTrigger Functions -------------------------------*/
    
ViStatus _VI_FUNC IviPower_ConfigureMeasurementTrigger (ViSession vi, ViInt32 triggerSource);

    /*- IviPowerMeasureMultiPoint Functions --------------------------------*/
    
ViStatus _VI_FUNC IviPower_ConfigureMultiPointMeasurement (ViSession vi, ViConstString channel,
                                                           ViInt32 measureFunction, ViInt32 sampleCount,
                                                           ViReal64 sampleOffset, ViReal64 sampleInterval);

ViStatus _VI_FUNC IviPower_FetchMultiPointMeasurement     (ViSession vi, ViConstString channel,
                                                           ViInt32 maxTime,
                                                           ViInt32 readingArraySize, ViReal64 readingArray[],
                                                           ViInt32 *pointsRead);

ViStatus _VI_FUNC IviPower_ReadMultiPointMeasurement      (ViSession vi, ViConstString channel,
                                                           ViInt32 maxTime,
                                                           ViInt32 readingArraySize, ViReal64 readingArray[],
                                                           ViInt32 *pointsRead);

/****************************************************************************
 *--------------- IviPower Class Error And Completion Codes ----------------*
 ****************************************************************************/

#define IVIPOWER_WARN_VOLTAGE_DECREASED             (IVI_CLASS_WARN_BASE + 1L)
#define IVIPOWER_WARN_CURRENT_DECREASED             (IVI_CLASS_WARN_BASE + 2L)

#define IVIPOWER_ERROR_OVER_VOLTAGE                 (IVI_CLASS_ERROR_BASE + 1L)
#define IVIPOWER_ERROR_OVER_CURRENT                 (IVI_CLASS_ERROR_BASE + 2L)
#define IVIPOWER_ERROR_OVER_TEMPERATURE             (IVI_CLASS_ERROR_BASE + 3L)

#define IVIPOWER_ERROR_INVALID_WAVEFORM             (IVI_CLASS_ERROR_BASE + 11L)
#define IVIPOWER_ERROR_INVALID_WAVEFORM_LENGTH      (IVI_CLASS_ERROR_BASE + 12L)
#define IVIPOWER_ERROR_NO_WAVEFORMS_AVAILABLE       (IVI_CLASS_ERROR_BASE + 13L)
#define IVIPOWER_ERROR_SAMPLE_OUT_OF_RANGE          (IVI_CLASS_ERROR_BASE + 14L)
#define IVIPOWER_ERROR_WAVEFORM_IN_USE              (IVI_CLASS_ERROR_BASE + 15L)
#define IVIPOWER_ERROR_WAVEFORM_OUT_OF_RANGE        (IVI_CLASS_ERROR_BASE + 16L)

#define IVIPOWER_ERROR_INVALID_TRANSIENT            (IVI_CLASS_ERROR_BASE + 21L)
#define IVIPOWER_ERROR_INVALID_TRANSIENT_LENGTH     (IVI_CLASS_ERROR_BASE + 22L)
#define IVIPOWER_ERROR_NO_TRANSIENTS_AVAILABLE      (IVI_CLASS_ERROR_BASE + 23L)
#define IVIPOWER_ERROR_TRANSIENT_IN_USE             (IVI_CLASS_ERROR_BASE + 24L)
#define IVIPOWER_ERROR_TRANSIENT_OUT_OF_RANGE       (IVI_CLASS_ERROR_BASE + 25L)

#define IVIPOWER_ERROR_MAX_TIME_EXCEEDED            (IVI_CLASS_ERROR_BASE + 26L)

#define IVIPOWER_WARNMSG_VOLTAGE_DECREASED          "Voltage decreased to satisfy current requirements."
#define IVIPOWER_WARNMSG_CURRENT_DECREASED          "Current decreased to satisfy voltage requirements."

#define IVIPOWER_ERRMSG_OVER_VOLTAGE                "Over-voltage detected."
#define IVIPOWER_ERRMSG_OVER_CURRENT                "Over-current detected."
#define IVIPOWER_ERRMSG_OVER_TEMPERATURE            "Over-temperature detected."

#define IVIPOWER_ERRMSG_INVALID_WAVEFORM            "Invalid waveform."
#define IVIPOWER_ERRMSG_INVALID_WAVEFORM_LENGTH     "Invalid waveform length."
#define IVIPOWER_ERRMSG_NO_WAVEFORMS_AVAILABLE      "No waveforms available."
#define IVIPOWER_ERRMSG_SAMPLE_OUT_OF_RANGE         "Waveform sample not in range [-1,+1]."
#define IVIPOWER_ERRMSG_WAVEFORM_IN_USE             "Waveform in use."
#define IVIPOWER_ERRMSG_WAVEFORM_OUT_OF_RANGE       "Waveform exceeds power supply capabilities."

#define IVIPOWER_ERRMSG_INVALID_TRANSIENT           "Invalid transient."
#define IVIPOWER_ERRMSG_INVALID_TRANSIENT_LENGTH    "Invalid transient length."  
#define IVIPOWER_ERRMSG_NO_TRANSIENTS_AVAILABLE     "No transients available."
#define IVIPOWER_ERRMSG_TRANSIENT_IN_USE            "Transient in use."
#define IVIPOWER_ERRMSG_TRANSIENT_OUT_OF_RANGE      "Transient exceeds power supply capabilities."

#define IVIPOWER_ERRMSG_MAX_TIME_EXCEEDED           "Max Time exceeded before "\
                                                    "the operation completed."

#define IVIPOWER_ERROR_CODES_AND_MSGS \
        {IVIPOWER_WARN_VOLTAGE_DECREASED,           IVIPOWER_WARNMSG_VOLTAGE_DECREASED},        \
        {IVIPOWER_WARN_CURRENT_DECREASED,           IVIPOWER_WARNMSG_CURRENT_DECREASED},        \
        {IVIPOWER_ERROR_OVER_VOLTAGE,               IVIPOWER_ERRMSG_OVER_VOLTAGE},              \
        {IVIPOWER_ERROR_OVER_CURRENT,               IVIPOWER_ERRMSG_OVER_CURRENT},              \
        {IVIPOWER_ERROR_OVER_TEMPERATURE,           IVIPOWER_ERRMSG_OVER_TEMPERATURE},          \
        {IVIPOWER_ERROR_INVALID_WAVEFORM,           IVIPOWER_ERRMSG_INVALID_WAVEFORM},          \
        {IVIPOWER_ERROR_INVALID_WAVEFORM_LENGTH,    IVIPOWER_ERRMSG_INVALID_WAVEFORM_LENGTH},   \
        {IVIPOWER_ERROR_NO_WAVEFORMS_AVAILABLE,     IVIPOWER_ERRMSG_NO_WAVEFORMS_AVAILABLE},    \
        {IVIPOWER_ERROR_SAMPLE_OUT_OF_RANGE,        IVIPOWER_ERRMSG_SAMPLE_OUT_OF_RANGE},       \
        {IVIPOWER_ERROR_WAVEFORM_IN_USE,            IVIPOWER_ERRMSG_WAVEFORM_IN_USE},           \
        {IVIPOWER_ERROR_WAVEFORM_OUT_OF_RANGE,      IVIPOWER_ERRMSG_WAVEFORM_OUT_OF_RANGE},     \
        {IVIPOWER_ERROR_INVALID_TRANSIENT,          IVIPOWER_ERRMSG_INVALID_TRANSIENT},         \
        {IVIPOWER_ERROR_INVALID_TRANSIENT_LENGTH,   IVIPOWER_ERRMSG_INVALID_TRANSIENT_LENGTH},  \
        {IVIPOWER_ERROR_NO_TRANSIENTS_AVAILABLE,    IVIPOWER_ERRMSG_NO_TRANSIENTS_AVAILABLE},   \
        {IVIPOWER_ERROR_TRANSIENT_IN_USE,           IVIPOWER_ERRMSG_TRANSIENT_IN_USE},          \
        {IVIPOWER_ERROR_TRANSIENT_OUT_OF_RANGE,     IVIPOWER_ERRMSG_TRANSIENT_OUT_OF_RANGE},    \
        {IVIPOWER_ERROR_MAX_TIME_EXCEEDED,          IVIPOWER_ERRMSG_MAX_TIME_EXCEEDED}

/****************************************************************************
 *---------------------------- End Include File ----------------------------*
 ****************************************************************************/

#if defined(__cplusplus) || defined(__cplusplus__)
}
#endif

#endif /* IVIPOWER_HEADER */

