/****************************************************************************
 *           California Instruments Cs Series AC Power Source                           
 *---------------------------------------------------------------------------
 *   Copyright (c) National Instruments 2002.  All Rights Reserved.         
 *---------------------------------------------------------------------------
 *                                                                          
 * Title:    ciCs.h                                        
 * Purpose:  California Instruments Cs Series AC Current Source                                       
 *           instrument driver declarations.                                
 *                                                                          
 ****************************************************************************/

#ifndef __CICS_HEADER
#define __CICS_HEADER

#include <ivi.h>

#if defined(__cplusplus) || defined(__cplusplus__)
extern "C" {
#endif

/****************************************************************************
 *----------------- Instrument Driver Revision Information -----------------*
 ****************************************************************************/
#define CICS_MAJOR_VERSION                1     /* Instrument driver major version   */
#define CICS_MINOR_VERSION                0     /* Instrument driver minor version   */
                                                                
#define CICS_CLASS_SPEC_MAJOR_VERSION     1     /* Class specification major version */
#define CICS_CLASS_SPEC_MINOR_VERSION     0     /* Class specification minor version */

#define CICS_SUPPORTED_INSTRUMENT_MODELS  "3000CS,4500CS,6000CS,9000CS,12000CS,13500CS,18000CS"
#define CICS_DRIVER_VENDOR                "California Instruments"
#define CICS_DRIVER_DESCRIPTION           "California Instruments Cs Series AC Current Source"

/**************************************************************************** 
 *------------------------------ Useful Macros -----------------------------* 
 ****************************************************************************/

#define CICS_VAL_MAX_TIME_INFINITE        (0L)
#define CICS_VAL_MAX_TIME_IMMEDIATE       (1L)

	/*- Defined values for rangeType parameter of function -----------------*/
    /*- ConfigureOutputRange -----------------------------------------------*/
#define CICS_VAL_RANGE_VOLTAGE			(0L)
#define CICS_VAL_RANGE_CURRENT			(1L)

	/*- Defined values for outputState parameter of function ---------------*/
	/*- QueryOutputState ---------------------------------------------------*/
#define CICS_VAL_OUTPUT_CONSTANT_VOLTAGE    	(0L)
#define CICS_VAL_OUTPUT_CONSTANT_CURRENT	    (1L)
#define CICS_VAL_OUTPUT_OVER_VOLTAGE           (2L)
#define CICS_VAL_OUTPUT_OVER_CURRENT	        (3L)
#define CICS_VAL_OUTPUT_UNREGULATED		    (4L)
#define CICS_VAL_OUTPUT_OVER_TEMPERATURE	    (5L)

	/*- Defined values for measurementType parameter of functions ----------*/
    /*- Measure ------------------------------------------------------------*/
	/*- Fetch --------------------------------------------------------------*/
#define CICS_VAL_MEASURE_AC_VOLTAGE_RMS		(0L)
#define CICS_VAL_MEASURE_AC_CURRENT_RMS		(1L)
#define CICS_VAL_MEASURE_VOLTAGE_MAXIMUM	    (2L)
#define CICS_VAL_MEASURE_VOLTAGE_CRESTFACTOR	(3L)
#define CICS_VAL_MEASURE_AC_POWER		        (4L)
#define CICS_VAL_MEASURE_AC_APPARENT_POWER	    (5L)
#define CICS_VAL_MEASURE_AC_REACTIVE_POWER	    (6L)
#define CICS_VAL_MEASURE_AC_TOTAL_POWER		(7L)
#define CICS_VAL_MEASURE_AC_POWER_FACTOR	    (8L)
#define CICS_VAL_MEASURE_NEUTRAL_AC_CURRENT_RMS	    (9)
#define CICS_VAL_MEASURE_FREQUENCY		        		(10L)
#define CICS_VAL_MEASURE_PHASE  		        		(11L)

	/*- Defined values for measurementType parameter of functions ----------*/
    /*- MeasureHarmonic ----------------------------------------------------*/
	/*- FetchHarmonic ------------------------------------------------------*/
#define CICS_VAL_HARMONIC_VOLTAGE_AMPLITUDE	        (18L)
#define CICS_VAL_HARMONIC_VOLTAGE_PHASE		        (19L)
#define CICS_VAL_HARMONIC_CURRENT_AMPLITUDE	        (20L)
#define CICS_VAL_HARMONIC_CURRENT_PHASE	            (21L)
#define CICS_VAL_HARMONIC_NEUTRAL_CURRENT_AMPLITUDE 	(22L)
#define CICS_VAL_HARMONIC_NEUTRAL_CURRENT_PHASE     	(23L)

	/*- Defined values for measurementType parameter of functions ----------*/
    /*- MeasureArray -------------------------------------------------------*/
	/*- FetchArray ---------------------------------------------------------*/
/* #define CICS_VAL_HARMONIC_VOLTAGE_AMPLITUDE				DEFINED ABOVE */
/* #define CICS_VAL_HARMONIC_VOLTAGE_PHASE					DEFINED ABOVE */
/* #define CICS_VAL_HARMONIC_CURRENT_AMPLITUDE				DEFINED ABOVE */
/* #define CICS_VAL_HARMONIC_CURRENT_PHASE					DEFINED ABOVE */
/* #define CICS_VAL_HARMONIC_NEUTRAL_CURRENT_AMPLITUDE		DEFINED ABOVE */
/* #define CICS_VAL_HARMONIC_NEUTRAL_CURRENT_PHASE			DEFINED ABOVE */
/* #define CICS_VAL_MEASURE_DC_VOLTAGE						DEFINED ABOVE */
/* #define CICS_VAL_MEASURE_DC_CURRENT						DEFINED ABOVE */
/* #define CICS_VAL_MEASURE_NEUTRAL_DC_CURRENT				DEFINED ABOVE */

/**************************************************************************** 
 *---------------------------- Attribute Defines ---------------------------* 
 ****************************************************************************/

    /*- IVI Inherent Instrument Attributes ---------------------------------*/    

        /* User Options */
#define CICS_ATTR_RANGE_CHECK                   IVI_ATTR_RANGE_CHECK                    /* ViBoolean */
#define CICS_ATTR_QUERY_INSTRUMENT_STATUS       IVI_ATTR_QUERY_INSTRUMENT_STATUS        /* ViBoolean */
#define CICS_ATTR_CACHE                         IVI_ATTR_CACHE                          /* ViBoolean */
#define CICS_ATTR_SIMULATE                      IVI_ATTR_SIMULATE                       /* ViBoolean */
#define CICS_ATTR_RECORD_COERCIONS              IVI_ATTR_RECORD_COERCIONS               /* ViBoolean */

#define	KPBOP_ATTR_DRIVER_SETUP                  IVI_ATTR_DRIVER_SETUP					/* ViString  */
#define KPBOP_ATTR_INTERCHANGE_CHECK             IVI_ATTR_INTERCHANGE_CHECK				/* ViBoolean */

        /* Instrument Capabilities */
#define CICS_ATTR_CHANNEL_COUNT                 IVI_ATTR_CHANNEL_COUNT                  /* ViInt32,  read-only  */

        /* Driver Information  */
#define CICS_ATTR_SPECIFIC_DRIVER_PREFIX        IVI_ATTR_SPECIFIC_DRIVER_PREFIX         /* ViString, read-only  */
#define CICS_ATTR_SUPPORTED_INSTRUMENT_MODELS   IVI_ATTR_SUPPORTED_INSTRUMENT_MODELS    /* ViString, read-only  */
#define CICS_ATTR_GROUP_CAPABILITIES            IVI_ATTR_GROUP_CAPABILITIES             /* ViString, read-only  */
#define CICS_ATTR_INSTRUMENT_MANUFACTURER       IVI_ATTR_INSTRUMENT_MANUFACTURER        /* ViString, read-only  */
#define CICS_ATTR_INSTRUMENT_MODEL              IVI_ATTR_INSTRUMENT_MODEL               /* ViString, read-only  */
#define CICS_ATTR_INSTRUMENT_FIRMWARE_REVISION  IVI_ATTR_INSTRUMENT_FIRMWARE_REVISION              /* ViString, read-only  */
#define CICS_ATTR_SPECIFIC_DRIVER_REVISION      IVI_ATTR_SPECIFIC_DRIVER_REVISION       /* ViString, read-only  */
#define CICS_ATTR_SPECIFIC_DRIVER_VENDOR        IVI_ATTR_SPECIFIC_DRIVER_VENDOR         /* ViString, read-only  */
#define CICS_ATTR_SPECIFIC_DRIVER_DESCRIPTION   IVI_ATTR_SPECIFIC_DRIVER_DESCRIPTION    /* ViString, read-only  */
#define CICS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MAJOR_VERSION IVI_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MAJOR_VERSION /* ViInt32, read-only */
#define CICS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MINOR_VERSION IVI_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MINOR_VERSION /* ViInt32, read-only */

        /* Advanced Session Information */
#define CICS_ATTR_LOGICAL_NAME                  IVI_ATTR_LOGICAL_NAME                   /* ViString, read-only  */
#define CICS_ATTR_IO_RESOURCE_DESCRIPTOR        IVI_ATTR_IO_RESOURCE_DESCRIPTOR         /* ViString, read-only  */
   

    /*- Instrument-Specific Attributes -------------------------------------*/
/*- IviACPwr Fundamental Attributes -*/
#define CICS_ATTR_OUTPUT_PHASE_COUNT	    (IVI_CLASS_PUBLIC_ATTR_BASE + 1L)           /* ViInt32 */
#define CICS_ATTR_OUTPUT_ENABLED		    (IVI_CLASS_PUBLIC_ATTR_BASE + 2L)	        /* ViBoolean */
#define CICS_ATTR_SLEW_FREQUENCY_RATE		(IVI_CLASS_PUBLIC_ATTR_BASE + 3L)           /* ViReal64 */
#define CICS_ATTR_FUNCTION   			    (IVI_CLASS_PUBLIC_ATTR_BASE + 4L)	        /* ViString */
#define CICS_ATTR_FREQUENCY			    	(IVI_CLASS_PUBLIC_ATTR_BASE + 5L)	        /* ViReal64 */
#define CICS_ATTR_CLIPPING_LEVEL		    (IVI_CLASS_PUBLIC_ATTR_BASE + 6L)	        /* ViReal64 */
#define CICS_ATTR_OUTPUT_PROTECTION_DELAY	(IVI_CLASS_PUBLIC_ATTR_BASE + 7L)	        /* ViReal64 */
#define CICS_ATTR_VOLTAGE_LIMIT_BEHAVIOR	(IVI_CLASS_PUBLIC_ATTR_BASE + 8L)	        /* ViInt32 */
#define CICS_ATTR_VOLTAGE_LIMIT      		(IVI_CLASS_PUBLIC_ATTR_BASE + 9L)	        /* ViReal64 */
#define CICS_ATTR_OVP_ENABLED 			    (IVI_CLASS_PUBLIC_ATTR_BASE + 10L)	        /* ViBoolean */
#define CICS_ATTR_OVP_LIMIT			    	(IVI_CLASS_PUBLIC_ATTR_BASE + 11L) 	        /* ViReal64 */
#define CICS_ATTR_CURRENT_LEVEL				(IVI_CLASS_PUBLIC_ATTR_BASE + 12L)	        /* ViReal64 */
#define CICS_ATTR_PHASE_ANGLE			    (IVI_CLASS_PUBLIC_ATTR_BASE + 13L)	        /* ViReal64 */
#define CICS_ATTR_SLEW_CURRENT_RATE	  		(IVI_CLASS_PUBLIC_ATTR_BASE + 14L)	        /* ViReal64 */
#define CICS_ATTR_VOLTAGE_RANGE_MINIMUM    	(IVI_CLASS_PUBLIC_ATTR_BASE + 15L)	        /* ViReal64 */
#define CICS_ATTR_VOLTAGE_RANGE_MAXIMUM    	(IVI_CLASS_PUBLIC_ATTR_BASE + 16L)	        /* ViReal64 */
#define CICS_ATTR_CURRENT_MAXIMUM	  	    (IVI_CLASS_PUBLIC_ATTR_BASE + 17L)	        /* ViReal64 */
#define CICS_ATTR_VOLTAGE_MAXIMUM	  	   	(IVI_CLASS_PUBLIC_ATTR_BASE + 18L)	        /* ViReal64 */
#define CICS_ATTR_OUTPUT_ALC_STATE	  	   	(IVI_CLASS_PUBLIC_ATTR_BASE + 19L)	        /* ViBoolean */
#define CICS_ATTR_FREQUENCY_MINIMUM	  		(IVI_CLASS_PUBLIC_ATTR_BASE + 20L)	        /* ViReal64 */ 
#define CICS_ATTR_FREQUENCY_MAXIMUM	  		(IVI_CLASS_PUBLIC_ATTR_BASE + 21L)	        /* ViReal64 */
#define CICS_ATTR_OUTPUT_PHASE_MODE	  		(IVI_CLASS_PUBLIC_ATTR_BASE + 22L)	        /* ViReal64 */ 

    /*- IviACPwr Extended Attributes -*/
        /*- IviACPwrArbWaveform Extension Group -*/
#define CICS_ATTR_ID_QUERY_RESPONSE       	(IVI_SPECIFIC_PUBLIC_ATTR_BASE + 1L)        /* ViString (Read Only) */
#define CICS_ATTR_INSTR_USER_TABLE_LIST    	(IVI_SPECIFIC_PUBLIC_ATTR_BASE + 2L)

#define CICS_ATTR_OUTPUT_MODE   			(IVI_SPECIFIC_PUBLIC_ATTR_BASE + 3L)	    /* ViString */

#define CICS_ATTR_MAX_NUM_WAVEFORMS        	(IVI_CLASS_PUBLIC_ATTR_BASE + 101L)	        /* ViInt32 */
#define CICS_ATTR_WAVEFORM_QUANTUM         	(IVI_CLASS_PUBLIC_ATTR_BASE + 102L)         /* ViInt32 */
#define CICS_ATTR_MIN_WAVEFORM_SIZE        	(IVI_CLASS_PUBLIC_ATTR_BASE + 103L) 	    /* ViInt32 */
#define CICS_ATTR_MAX_WAVEFORM_SIZE        	(IVI_CLASS_PUBLIC_ATTR_BASE + 104L) 	    /* ViInt32 */
        
/*- IviACPwrTrigger Extension Group -*/
#define CICS_ATTR_TRIGGER_SOURCE	          			(IVI_CLASS_PUBLIC_ATTR_BASE + 201L)	    /* ViInt32 */
#define CICS_ATTR_TRIGGER_DELAY		          			(IVI_CLASS_PUBLIC_ATTR_BASE + 202L)     /* ViReal64 */
#define CICS_ATTR_OUTPUT_TRIGGER_ENABLED        		(IVI_CLASS_PUBLIC_ATTR_BASE + 203L)	    /* ViBoolean */
#define CICS_ATTR_OUTPUT_TRIGGER_SOURCE		    		(IVI_CLASS_PUBLIC_ATTR_BASE + 204L)	    /* ViInt32 */
#define CICS_ATTR_TRIGGERED_CURRENT_MODE 	    		(IVI_CLASS_PUBLIC_ATTR_BASE + 205L)	    /* ViInt32 */   
#define CICS_ATTR_TRIGGERED_FREQUENCY_MODE	    		(IVI_CLASS_PUBLIC_ATTR_BASE + 206L)	    /* ViInt32 */
#define CICS_ATTR_TRIGGERED_FUNCTION_MODE	    		(IVI_CLASS_PUBLIC_ATTR_BASE + 207L)	    /* ViInt32 */
#define CICS_ATTR_TRIGGERED_PHASE_ANGLE_MODE    		(IVI_CLASS_PUBLIC_ATTR_BASE + 208L)	    /* ViInt32 */
#define CICS_ATTR_TRIGGERED_SLEW_CURRENT_RATE_MODE 		(IVI_CLASS_PUBLIC_ATTR_BASE + 209L)		/* ViInt32 */
#define CICS_ATTR_TRIGGERED_SLEW_FREQUENCY_RATE_MODE 	(IVI_CLASS_PUBLIC_ATTR_BASE + 210L) 	/* ViReal64 */  
#define CICS_ATTR_TRIGGERED_CURRENT_LEVEL       		(IVI_CLASS_PUBLIC_ATTR_BASE + 211L)	    /* ViReal64 */
#define CICS_ATTR_TRIGGERED_FREQUENCY           		(IVI_CLASS_PUBLIC_ATTR_BASE + 212L)     /* ViReal64 */
#define CICS_ATTR_TRIGGERED_FUNCTION            		(IVI_CLASS_PUBLIC_ATTR_BASE + 213L)	    /* ViReal64 */
#define CICS_ATTR_TRIGGERED_PHASE_ANGLE         		(IVI_CLASS_PUBLIC_ATTR_BASE + 214L)	    /* ViReal64 */ 
#define CICS_ATTR_TRIGGERED_SLEW_CURRENT_RATE   		(IVI_CLASS_PUBLIC_ATTR_BASE + 215L)	    /* ViReal64 */ 
#define CICS_ATTR_TRIGGERED_SLEW_FREQUENCY_RATE 		(IVI_CLASS_PUBLIC_ATTR_BASE + 216L)     /* ViInt32 */

   
        /*- IviACPwrPulseTrigger Extension Group -*/
#define CICS_ATTR_TRIGGER_PULSE_COUNT       (IVI_CLASS_PUBLIC_ATTR_BASE + 301L)	        /* ViInt32 */
#define CICS_ATTR_TRIGGER_PULSE_WIDTH       (IVI_CLASS_PUBLIC_ATTR_BASE + 302L)	        /* ViReal64 */
#define CICS_ATTR_TRIGGER_PULSE_PERIOD      (IVI_CLASS_PUBLIC_ATTR_BASE + 303L)	        /* ViReal64 */

        /*- IviACPwrListTrigger Extension Group -*/
#define CICS_ATTR_TRIGGER_LIST_COUNT        (IVI_CLASS_PUBLIC_ATTR_BASE + 401L)         /* ViInt32 */
#define CICS_ATTR_TRIGGER_LIST_MODE         (IVI_CLASS_PUBLIC_ATTR_BASE + 402L)	        /* ViInt32 */

        /*- IviACPwrSynchronization Extension Group -*/
#define CICS_ATTR_TRIGGER_SYNCHRONIZATION_SOURCE (IVI_CLASS_PUBLIC_ATTR_BASE + 501L)	/* ViInt32 */
#define CICS_ATTR_TRIGGER_SYNCHRONIZATION_PHASE  (IVI_CLASS_PUBLIC_ATTR_BASE + 502L)	/* ViReal64 */

        /*- IviACPwrAcquisition Extension Group -*/
#define CICS_ATTR_ACQUISITION_TRIGGER_SOURCE     	(IVI_CLASS_PUBLIC_ATTR_BASE + 601L)	/* ViInt32 */
#define CICS_ATTR_ACQUISITION_START_TIME         	(IVI_CLASS_PUBLIC_ATTR_BASE + 602L)	/* ViReal64 */
#define CICS_ATTR_ACQUISITION_TIME_INTERVAL      	(IVI_CLASS_PUBLIC_ATTR_BASE + 603L)	/* ViReal64 */ 
#define CICS_ATTR_WINDOW_TYPE				      	(IVI_CLASS_PUBLIC_ATTR_BASE + 604L)   /* ViInt32 */

/**************************************************************************** 
 *------------------------ Attribute Value Defines -------------------------* 
 ****************************************************************************/

        /* Instrument specific attribute value definitions */

    /*- Defined values for CICS_ATTR_OUTPUT_PHASE_COUNT -*/
#define CICS_VAL_1_PHASE               	(1L)
#define CICS_VAL_3_PHASE               	(3L)
#define CICS_VAL_1_PHASE_ANGLE       	(0L)
#define CICS_VAL_3_PHASE_ANGLE       	(120L)
	
/*- Defined values for CICS_ATTR_OUTPUT_ALC_STATE -*/
#define CICS_VAL_ALC_OFF		            0
#define CICS_VAL_ALC_ON 	                1
#define CICS_VAL_ALC_REGULATE               2

	    /*- Defined values for CICS_ATTR_VOLTAGE_LIMIT_BEHAVIOR -*/
#define CICS_VAL_VOLTAGE_REGULATE           0
#define CICS_VAL_VOLTAGE_TRIP               1

		/*- Defined values for CICS_ATTR_SLEW_CURRENT_RATE -*/
#define CICS_VAL_INFINTY_SLEW_RATE			(-1.0)

		/*- Defined values for CICS_ATTR_FUNCTION -*/
#define CICS_VAL_SINE					"SINUSOID"
#define CICS_VAL_SQUARE					"SQUARE"
#define CICS_VAL_CLIPPED_SINE			"CSINUSOID"

		/*- Defined values for CICS_ATTR_TRIGGER_SOURCE -*/
#define CICS_VAL_TRIG_IMMEDIATE          (0L)
#define CICS_VAL_TRIG_EXTERNAL           (1L)
#define CICS_VAL_SOFTWARE_TRIG           (2L)

		/*- Defined values for CICS_ATTR_TRIGGERED_VOLTAGE_MODE -*/
		/*- Defined values for CICS_ATTR_TRIGGERED_FREQUENCY_MODE -*/
		/*- Defined values for CICS_ATTR_TRIGGERED_FUNCTION_MODE -*/
		/*- Defined values for CICS_ATTR_TRIGGERED_PHASE_ANGLE_MODE -*/
		/*- Defined values for CICS_ATTR_TRIGGERED_FREQUENCY_MODE -*/
#define CICS_VAL_TRIGGER_MODE_FIX      (0L)
#define CICS_VAL_TRIGGER_MODE_STEP     (1L)
#define CICS_VAL_TRIGGER_MODE_PULSE    (2L)
#define CICS_VAL_TRIGGER_MODE_LIST     (3L)

		/*- Defined values for CICS_ATTR_TRIGGER_LIST_MODE -*/
#define CICS_VAL_TRIGGER_LIST_STEP_ONCE	  (0L)
#define CICS_VAL_TRIGGER_LIST_STEP_AUTO	  (1L)

		/*- Defined values for CICS_ATTR_TRIGGER_SYNCHRONIZATION_SOURCE -*/
#define CICS_VAL_SYNCHRONIZATION_SOURCE_IMMEDIATE	(1L)
#define CICS_VAL_SYNCHRONIZATION_SOURCE_PHASE		(2L)
	
	/*- Defined values for CICS_ATTR_ACQUISITION_TRIGGER_SOURCE -*/
/* #define CICS_VAL_TRIG_EXTERNAL           DEFINED ABOVE */
/* #define CICS_VAL_SOFTWARE_TRIG           DEFINED ABOVE */
#define CICS_VAL_TRIG_TTLT		(21L)

		/*- Defined values for CICS_ATTR_OUTPUT_TRIGGER_SOURCE -*/
#define CICS_VAL_OUTPUT_TRIGGER_SOURCE_BOT		(0L)
#define CICS_VAL_OUTPUT_TRIGGER_SOURCE_EOT		(1L)
#define CICS_VAL_OUTPUT_TRIGGER_SOURCE_LIST	(2L)

		/*- Defined values for CICS_ATTR_WINDOW_TYPE -*/
#define CICS_VAL_WINDOW_TYPE_RECTANGULAR	(0L)
#define CICS_VAL_WINDOW_TYPE_KBESSEL		(0L)


/*****************************************************************************
 *---------------------------- Useful Macros --------------------------------*
 *****************************************************************************/
    /*- Defined values for CICS_ATTR_INSTR_PHASE_COUPLING -*/
#define CICS_VAL_COUPLING_NONE              0
#define CICS_VAL_COUPLING_ALL               1


/**************************************************************************** 
 *---------------- Instrument Driver Function Declarations -----------------* 
 ****************************************************************************/

    /*- Init and Close Functions -------------------------------------------*/
ViStatus _VI_FUNC  ciCs_init (
	ViRsrc		resourceName,
	ViBoolean	IDQuery,
	ViBoolean	resetDevice,
	ViSession*	vi,
	ViUInt32 	baudRate
);

ViStatus _VI_FUNC  ciCs_InitWithOptions (
	ViRsrc			resourceName,
	ViBoolean		IDQuery,
	ViBoolean		resetDevice,
	ViConstString	optionString,
	ViSession*		newVi,
	ViUInt32 	baudRate
);

ViStatus _VI_FUNC  ciCs_close ( ViSession vi);   

    /*- Coercion Info Functions --------------------------------------------*/
ViStatus _VI_FUNC  ciCs_GetNextCoercionRecord (
	ViSession	vi,
	ViInt32		bufferSize,
	ViChar		record[]
);

    /*- Locking Functions --------------------------------------------------*/
ViStatus _VI_FUNC  ciCs_LockSession (ViSession vi, ViBoolean *callerHasLock);   
ViStatus _VI_FUNC  ciCs_UnlockSession (ViSession vi, ViBoolean *callerHasLock);

        
	/*- California Instrument functions ------------------------------------*/
ViStatus _VI_FUNC ciCs_ConfigureOutput (
	ViSession	vi,
	ViInt32		numberOfPhases
);
ViStatus _VI_FUNC ciCs_ConfigureOutputEnabled (
	ViSession	vi,
	ViBoolean	enabled
);
ViStatus _VI_FUNC ciCs_ConfigureOutputALCState (
	ViSession	vi,
	ViInt32		enabled
);
ViStatus _VI_FUNC ciCs_ConfigureOutputRange (
	ViSession		vi,
	ViReal64		range
);
ViStatus _VI_FUNC ciCs_ConfigureOutputPhaseMode (
	ViSession	vi,
	ViInt32		phaseMode
);
	
ViStatus _VI_FUNC ciCs_ConfigureVoltageLimit (
	ViSession		io,
	ViConstString	phaseName,
	ViInt32			behavior,
	ViReal64		limit
);
ViStatus _VI_FUNC ciCs_ConfigureOVP (
	ViSession		vi,
	ViConstString	phaseName,
	ViBoolean		enabled,
	ViReal64		limit
);
ViStatus _VI_FUNC ciCs_ConfigureCurrentLevel (
	ViSession		vi,
	ViConstString	phaseName,
	ViReal64		level
);
ViStatus _VI_FUNC ciCs_ConfigureFrequencyValue (
    ViSession       vi,
    ViReal64        value
);
ViStatus _VI_FUNC ciCs_ConfigurePhaseAngle (
	ViSession		vi,
	ViConstString	phaseName,
	ViReal64		angle
);
ViStatus _VI_FUNC ciCs_ConfigureWaveform (
	ViSession		vi,
	ViConstString	function,
	ViReal64		frequency
);
ViStatus _VI_FUNC ciCs_ConfigureClippingLevel (
	ViSession	vi,
	ViReal64	clippingLevel
);
ViStatus _VI_FUNC ciCs_ConfigureSlewCurrentLevel (
	ViSession		vi,
	ViConstString	phaseName,
	ViReal64		slewVoltageRate
);
ViStatus _VI_FUNC ciCs_ConfigureSlewFrequency (
	ViSession	vi,
	ViReal64	slewFrequencyRate
);
ViStatus _VI_FUNC ciCs_GetPhaseName (
	ViSession	vi,
	ViInt32		index,
	ViInt32		bufferSize,
	ViChar _VI_FAR phaseName[]
);
ViStatus _VI_FUNC ciCs_WriteArbWaveform (
	ViSession		vi,
	ViConstString	name,
	ViInt32			waveformSize,
	ViReal64*		waveformDataArray
);
ViStatus _VI_FUNC ciCs_QueryArbWaveformCapabilities (
	ViSession	vi,
	ViInt32*	maxNbOfWaveforms,
	ViInt32*	waveformQuantum,
	ViInt32*	minWaveformSize,
	ViInt32*	maxWaveformSize
);
ViStatus _VI_FUNC ciCs_ClearAllArbWaveforms (
	ViSession	vi
);

ViStatus _VI_FUNC ciCs_ClearArbWaveforms (
    ViSession   vi,
    ViString waveformName
);

ViStatus _VI_FUNC ciCs_QueryDefinedWaveforms (
    ViSession       vi,
    ViChar          definedWaveforms[]
);

ViStatus _VI_FUNC ciCs_ConfigureAcqTriggerSource (
	ViSession	vi,
	ViInt32		source
);
ViStatus _VI_FUNC ciCs_ConfigureAcquisitionStartTime (
	ViSession	vi,
	ViReal64	startTime
);
ViStatus _VI_FUNC ciCs_ConfigureTriggerSource (
	ViSession	vi,
	ViInt32		source
);
ViStatus _VI_FUNC ciCs_ConfigureTriggerDelay (
	ViSession	vi,
	ViReal64	triggerDelay
);
ViStatus _VI_FUNC ciCs_ConfigureTTLTriggerOutput (
	ViSession	vi,
	ViBoolean	state,
	ViInt32		source
);
ViStatus _VI_FUNC ciCs_ConfigureTriggeredCurrentMode (
	ViSession		vi,
	ViConstString	phaseName,
	ViInt32			transientMode
);
ViStatus _VI_FUNC ciCs_ConfigureTriggeredFrequencyMode (
	ViSession		vi,
	ViInt32			transientMode
);
ViStatus _VI_FUNC ciCs_ConfigureTriggeredFunctionMode (
	ViSession		vi,
	ViInt32			transientMode
);
ViStatus _VI_FUNC ciCs_ConfigureTriggeredAngleMode (
	ViSession		vi,
	ViConstString	phaseName,
	ViInt32			transientMode
);
ViStatus _VI_FUNC ciCs_ConfigureTrigSlewVoltageMode (
	ViSession		vi,
	ViConstString	phaseName,
	ViInt32			transientMode
);
ViStatus _VI_FUNC ciCs_ConfigureTrigSlewFrequencyMode (
	ViSession		vi,
	ViInt32			transientMode
);
ViStatus _VI_FUNC ciCs_ConfigureTriggeredVoltageLevel (
	ViSession		vi,
	ViConstString	phaseName,
	ViReal64		triggeredLevel
);
ViStatus _VI_FUNC ciCs_ConfigureTriggeredFrequency (
	ViSession		vi,
	ViReal64		triggeredFrequency
);
ViStatus _VI_FUNC ciCs_ConfigureTriggeredFunction (
	ViSession		vi,
	ViConstString	triggeredFunction
);
ViStatus _VI_FUNC ciCs_ConfigureTriggeredAngle (
	ViSession		vi,
	ViConstString	phaseName,
	ViReal64		angle
);
ViStatus _VI_FUNC ciCs_ConfigureTriggeredSlewVoltage (
	ViSession		vi,
	ViConstString	phaseName,
	ViReal64		slewRateVoltage
);
ViStatus _VI_FUNC ciCs_ConfigureTriggeredSlewFrequency (
	ViSession		vi,
	ViReal64		slewRateFrequency
);
ViStatus _VI_FUNC ciCs_ConfigurePulse (
	ViSession	vi,
	ViInt32		count,
	ViReal64	width,
	ViReal64	period
);
ViStatus _VI_FUNC ciCs_ConfigureTriggerList (
	ViSession		vi,
	ViInt32			repeatCount,
	ViInt32			stepMode,
	ViInt32			listSize,
	ViReal64 _VI_FAR dwellTimes[],
	ViInt32  _VI_FAR repeatTimes[]
);
ViStatus _VI_FUNC ciCs_ConfigureTriggerVoltageList (
	ViSession		vi,
	ViConstString	phaseName,
	ViInt32			listSize,
	ViReal64 _VI_FAR voltage[]
);
ViStatus _VI_FUNC ciCs_ConfigureTriggerFrequencyList (
	ViSession	io,
	ViInt32		listSize,
	ViReal64 _VI_FAR frequency[]
);
ViStatus _VI_FUNC ciCs_ConfigureTriggerFunctionList (
	ViSession		vi,
	ViConstString	functionsList
);
ViStatus _VI_FUNC ciCs_ConfigureTriggerAngleList (
	ViSession		vi,
	ViConstString	phaseName,
	ViInt32			listSize,
	ViReal64 _VI_FAR angles[]
);
ViStatus _VI_FUNC ciCs_ConfigureTrigSlewVoltageList (
	ViSession		vi,
	ViConstString	phaseName,
	ViInt32			listSize,
	ViReal64 _VI_FAR slewVoltage[]
);
ViStatus _VI_FUNC ciCs_ConfigureTrigSlewFrequencyList (
	ViSession		vi,
	ViInt32			listSize,
	ViReal64 _VI_FAR slewFrequency[]
);
ViStatus _VI_FUNC ciCs_ConfigureTriggerTTLOutputList (
	ViSession		vi,
	ViInt32			listSize,
	ViBoolean _VI_FAR TTLOutput[]
);
ViStatus _VI_FUNC  ciCs_QueryTrnsListStatus (
	ViSession	vi,
	ViChar		trigStatus[]
);

ViStatus _VI_FUNC ciCs_ConfigureSynchronizationSource (
	ViSession	vi,
	ViInt32		source
);
ViStatus _VI_FUNC ciCs_ConfigureSynchronizationPhase (
	ViSession	vi,
	ViReal64	angle
);
ViStatus _VI_FUNC ciCs_SendSoftwareTrigger (
	ViSession	vi
);
ViStatus _VI_FUNC ciCs_QueryMaxCurrentLimit (
	ViSession		vi,
	ViReal64*		maxCurrentLimit
);
ViStatus _VI_FUNC ciCs_QueryMaxVoltageLevel (
	ViSession		vi,
	ViReal64*		maxVoltageLvel
);
ViStatus _VI_FUNC ciCs_QueryOutputState (
	ViSession		vi,
	ViConstString	phaseName,
	ViInt32			outputState,
	ViBoolean*		inState
);
ViStatus _VI_FUNC ciCs_ResetOutputProtection (
	ViSession		vi,
	ViConstString	phaseName
);
ViStatus _VI_FUNC ciCs_Measure (
	ViSession		vi,
	ViConstString	phaseName,
	ViInt32			maxTimeout,
	ViInt32			measurementType,
	ViReal64*		measurement
);
ViStatus _VI_FUNC ciCs_Fetch (
	ViSession		vi,
	ViConstString	phase,
	ViInt32			measurementType,
	ViReal64*		measurement
);
ViStatus _VI_FUNC ciCs_MeasureHarmonic (
	ViSession		vi,
	ViConstString	phaseName,
	ViInt32			timeout,
	ViInt32			harmonic,
	ViInt32			measurementType,
	ViReal64*		measurement
);
ViStatus _VI_FUNC ciCs_FetchHarmonic (
	ViSession		vi,
	ViConstString	phaseName,
	ViInt32			harmonic,
	ViInt32			measurementType,
	ViReal64*		measurement
);
ViStatus _VI_FUNC ciCs_MeasureArray (
	ViSession		vi,
	ViConstString	phaseName,
	ViInt32			timeOut,
	ViInt32			measurementType,
	ViInt32			arraySize,
	ViReal64		measurement[],
	ViInt32*		numberOfMeasurements
);
ViStatus _VI_FUNC ciCs_FetchArray (
	ViSession		vi,
	ViConstString	phaseName,
	ViInt32			measurementType,
	ViInt32			arraySize,
	ViReal64		measurement[],
	ViInt32*		numberOfMeasurements
);
ViStatus _VI_FUNC ciCs_InitiateTransient (
	ViSession vi
);
ViStatus _VI_FUNC ciCs_InitiateAcquisition (
	ViSession	vi
);
ViStatus _VI_FUNC ciCs_Abort (
	ViSession	vi
);

ViStatus _VI_FUNC  ciCs_StoreRecallRegister (
	ViSession vi,
    ViBoolean command,
    ViInt16 registers
);
                                             
    /*- Error Functions ----------------------------------------------------*/
ViStatus _VI_FUNC  ciCs_error_query (
	ViSession	vi,
	ViInt32*	errorCode,
	ViChar		errorMessage[]
);

ViStatus _VI_FUNC ciCs_GetError (
	ViSession	vi, 
	ViStatus*	errorCode, 
	ViInt32		bufferSize,
	ViChar		description[]
);
ViStatus _VI_FUNC ciCs_ClearError (ViSession vi);

ViStatus _VI_FUNC  ciCs_error_message (
	ViSession	vi,
	ViStatus	errorCode,
	ViChar		errorMessage[256]
);
	/*- Interchange Warning Functions --------------------------------------*/
ViStatus _VI_FUNC ciCs_GetNextInterchangeWarning (
	ViSession	vi, 
	ViInt32		bufferSize, 
	ViChar		warnString[]
);
ViStatus _VI_FUNC ciCs_ResetInterchangeCheck (ViSession vi);
ViStatus _VI_FUNC ciCs_ClearInterchangeWarnings (ViSession vi);

    /*- Utility Functions --------------------------------------------------*/
ViStatus _VI_FUNC  ciCs_reset (ViSession vi);
ViStatus _VI_FUNC  ciCs_ResetWithDefaults (ViSession vi);
ViStatus _VI_FUNC  ciCs_Disable (ViSession vi);
ViStatus _VI_FUNC  ciCs_InvalidateAllAttributes (ViSession vi);
ViStatus _VI_FUNC  ciCs_self_test (
	ViSession	vi,
	ViInt16*	selfTestResult,
	ViChar		selfTestMessage[]
);
ViStatus _VI_FUNC  ciCs_revision_query (
	ViSession	vi, 
	ViChar		instrumentDriverRevision[],
	ViChar		firmwareRevision[]
);
ViStatus _VI_FUNC ciCs_ReadInstrData (
	ViSession	vi,
	ViInt32		numBytes,
	ViChar		rdBuf[],
	ViInt32*	bytesRead
);
ViStatus _VI_FUNC ciCs_WriteInstrData (
	ViSession		vi,
	ViConstString	writeBuffer
);

    /*- Set, Get, and Check Attribute Functions ----------------------------*/
ViStatus _VI_FUNC  ciCs_GetAttributeViInt32 (ViSession vi, ViConstString phaseName, ViAttr attribute, ViInt32 *value);
ViStatus _VI_FUNC  ciCs_GetAttributeViReal64 (ViSession vi, ViConstString phaseName, ViAttr attribute, ViReal64 *value);
ViStatus _VI_FUNC  ciCs_GetAttributeViString (ViSession vi, ViConstString phaseName, ViAttr attribute, ViInt32 bufSize, ViChar value[]); 
ViStatus _VI_FUNC  ciCs_GetAttributeViSession (ViSession vi, ViConstString phaseName, ViAttr attribute, ViSession *value);
ViStatus _VI_FUNC  ciCs_GetAttributeViBoolean (ViSession vi, ViConstString phaseName, ViAttr attribute, ViBoolean *value);

ViStatus _VI_FUNC  ciCs_SetAttributeViInt32 (ViSession vi, ViConstString phaseName, ViAttr attribute, ViInt32 value);
ViStatus _VI_FUNC  ciCs_SetAttributeViReal64 (ViSession vi, ViConstString phaseName, ViAttr attribute, ViReal64 value);
ViStatus _VI_FUNC  ciCs_SetAttributeViString (ViSession vi, ViConstString phaseName, ViAttr attribute, ViConstString value); 
ViStatus _VI_FUNC  ciCs_SetAttributeViSession (ViSession vi, ViConstString phaseName, ViAttr attribute, ViSession value);
ViStatus _VI_FUNC  ciCs_SetAttributeViBoolean (ViSession vi, ViConstString phaseName, ViAttr attribute, ViBoolean value);

ViStatus _VI_FUNC  ciCs_CheckAttributeViInt32 (ViSession vi, ViConstString phaseName, ViAttr attribute, ViInt32 value);
ViStatus _VI_FUNC  ciCs_CheckAttributeViReal64 (ViSession vi, ViConstString phaseName, ViAttr attribute, ViReal64 value);
ViStatus _VI_FUNC  ciCs_CheckAttributeViString (ViSession vi, ViConstString phaseName, ViAttr attribute, ViConstString value); 
ViStatus _VI_FUNC  ciCs_CheckAttributeViSession (ViSession vi, ViConstString phaseName, ViAttr attribute, ViSession value);
ViStatus _VI_FUNC  ciCs_CheckAttributeViBoolean (ViSession vi, ViConstString phaseName, ViAttr attribute, ViBoolean value);

    /*********************************************************
        Functions reserved for class driver use only.
        End-users should not call these functions.  
     *********************************************************/
ViStatus _VI_FUNC  ciCs_IviInit (
	ViRsrc		resourceName,
	ViBoolean	IDQuery, 
	ViBoolean	reset,
	ViSession	vi,
	ViUInt32 	baudRate
);
ViStatus _VI_FUNC  ciCs_IviClose (ViSession vi);   

/****************************************************************************
 *------------------------ Error And Completion Codes ----------------------*
 ****************************************************************************/
#define CICS_ERROR_MAX_TIME_EXCEEDED             (IVI_CLASS_ERROR_BASE + 0x0001L)
#define CICS_ERROR_TRIGGER_NOT_SOFTWARE          (IVI_CLASS_ERROR_BASE + 0x0002L)


/****************************************************************************
 *--------- Obsolete Inherent Instrument Attributes and functions ----------*
 ****************************************************************************/
    /*- These attributes and functions have been deprecated and may not
        be supported in future versions of this driver. -*/

#define CICS_ATTR_SPECIFIC_DRIVER_MAJOR_VERSION IVI_ATTR_SPECIFIC_DRIVER_MAJOR_VERSION  /* ViInt32,  read-only  */
#define CICS_ATTR_SPECIFIC_DRIVER_MINOR_VERSION IVI_ATTR_SPECIFIC_DRIVER_MINOR_VERSION  /* ViInt32,  read-only  */

        /* Error Info */
#define CICS_ATTR_PRIMARY_ERROR                 IVI_ATTR_PRIMARY_ERROR                  /* ViInt32   */
#define CICS_ATTR_SECONDARY_ERROR               IVI_ATTR_SECONDARY_ERROR                /* ViInt32   */
#define CICS_ATTR_ERROR_ELABORATION             IVI_ATTR_ERROR_ELABORATION              /* ViString  */

        /* Advanced Session Information */
#define CICS_ATTR_IO_SESSION_TYPE               IVI_ATTR_IO_SESSION_TYPE                /* ViString, read-only  */
#define CICS_ATTR_IO_SESSION                    IVI_ATTR_IO_SESSION                     /* ViSession, read-only */
		
		/*- Deprecated Attribute Identifiers for Renamed Attributes -*/
#define CICS_ATTR_NUM_CHANNELS                  IVI_ATTR_CHANNEL_COUNT               /* ViInt32,  read-only  */
#define CICS_ATTR_RESOURCE_DESCRIPTOR           IVI_ATTR_IO_RESOURCE_DESCRIPTOR      /* ViString, read-only  */
#define CICS_ATTR_QUERY_INSTR_STATUS            IVI_ATTR_QUERY_INSTRUMENT_STATUS		/* ViBoolean */

		/*- Deprecated Error Information functions -*/
ViStatus _VI_FUNC  ciCs_GetErrorInfo (
	ViSession vi,
	ViStatus *primaryError,
	ViStatus *secondaryError, 
	ViChar errorElaboration[256]
);
ViStatus _VI_FUNC  ciCs_ClearErrorInfo (ViSession vi);

/**************************************************************************** 
 *---------------------------- End Include File ----------------------------* 
 ****************************************************************************/
#if defined(__cplusplus) || defined(__cplusplus__)
}
#endif
#endif /* __CICS_HEADER */




