#include <utility.h>
#include <gpib.h>
#include <rs232.h>
#include <ansi_c.h>
#include <formatio.h>
#include <cvirte.h>		/* Needed if linking in external compiler; harmless otherwise */
#include <userint.h>
#include "ciCs_example.h"
#include "ciCs.h"

static ViSession ciCs;
static ViStatus error_status, gpibOnLine, rs232OnLine, lanOnLine;

static ViChar waveform[600], waveform_output[12], source_mode[4], wave[20];  
static ViChar bufData[256], queryMaxCurrentStr[256];

static ViBoolean output_relay, queryOutputState;

static ViReal64 data[1024], voltarray[5000];     
static ViReal64 voltage, dc_offset, current, power, phaseangle, powerfactor;
static ViReal64 frequency, curr_delay, phaseangle, app_pow, peak_curr, crest_fact;
static ViReal64 dc_offset, line_line, freq_low_limit, freq_high_limit;
static ViReal64 current_limit, voltage_limit, voltagerange, currentrange;
static ViReal64 freq_value, volt_value, offset_value, curr_value, rang_value;
static ViReal64 volt_query, freq_query, curr_query, offset_query, volt_meas;
static ViReal64 power_meas, curr_meas, phase_mode, phase_angle, volt_mode;
static ViReal64 queryOutFuncDelay = 0.020, queryMinFreq, queryMaxFreq, queryMaxVoltage, queryMaxCurrent;
static ViReal64 queryVoltRange, queryFreq, queryPhaseAngle, queryVoltage, queryCurrent;

static ViInt16  y, n, index, numElement, i, queryPhaseMode;

static ViInt32 panelStar, panelSt, panelSteady, panelStart, panelMeas, panelHarm;
static ViInt32 checkA, checkB, checkC, clip_level, meas_volt, meas_current;
static ViInt32 meas_phase, meas_power, meas_app_pow, meas_pfactor, meas_peak_curr;
static ViInt32 meas_crest_fact, meas_dc_offset, meas_line_line, meas_check;
static ViInt32 stop_check, relay, range_button, remoteonoff, measure_check;
static ViInt32 display_mode, gpib_option, mode_option, register_status;
static ViInt32 register_number, address, device, comport, baud_rate;
static ViInt32 serial_flag, GPIBError, RS232Error, port_open, status;
static ViInt32 address, port1, port2, port3, port4, baudRate, baudRate_number, comPortNo;
static ViInt32 CntNumber, ACheck, BCheck,CCheck;
static ViChar ipString[100];
static ViChar ipPortNumber[100];

int maxitems, numitems, checked;
static char resource_Name[100];
static char int_message[120];

/****************************************************************/
void DisplayGPIBError (void);
void DisplayRS232Error (void);
void Error_check (char *error_message, int error_val);
void Query_display(void);
void collect_data (ViReal64 data[]);
void display_waveform (void);
/****************************************************************/ 
void BuildErrorString (ViSession ciLxs, ViStatus error, ViString errStr);


int main (int argc, char *argv[])
{
	if (InitCVIRTE (0, argv, 0) == 0)	/* Needed if linking in external compiler; harmless otherwise */
		return -1;	/* out of memory */
	
	if ((panelStart = LoadPanel (0, "ciCs_example.uir", PANEL_STAR)) < 0)
		
		Error_check ("Error Loading Main Window", panelStart);
	
	DisplayPanel (panelStart);
	RunUserInterface ();
	return 0;
}

/*********************************** INTERFACE CONTROL ***********************************/
int CVICALLBACK verifyConnection (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	ViStatus    error = VI_SUCCESS;  
	
	ViChar instrRev[512], driverRev[512];
	unsigned char iDent[512];
	
	switch (event) {
		case EVENT_COMMIT:
			/* Initialize and collect all limits */
			status = SetMouseCursor (VAL_HOUR_GLASS_CURSOR);
			GetCtrlVal(panelStart, PANEL_STAR_GPIB, &gpibOnLine);
			GetCtrlVal(panelStart, PANEL_STAR_RS232, &rs232OnLine);
			GetCtrlVal(panelStart, PANEL_STAR_LAN, &lanOnLine);
			
			voltage_limit = 0;
			if( gpibOnLine == 1 ){
			  	GetCtrlVal( panelStart, PANEL_STAR_GPIBADDRESS, &address);
			   	DisableBreakOnLibraryErrors();

        	   	if( ibsta & 0x8000 ){ 
        	      	status = SetMouseCursor (VAL_DEFAULT_CURSOR); 
        	      	DisplayGPIBError();
        	      	
        	      	return -1;
        	    }
        	   	else{
                  	Fmt(resource_Name, "%s<GPIB::%i::INSTR", address);
                }	
          	}
			else if( lanOnLine == 1 ){
			   	DisableBreakOnLibraryErrors();

        	   	if( ibsta & 0x8000 ){ 
        	      	status = SetMouseCursor (VAL_DEFAULT_CURSOR); 
        	      	DisplayGPIBError();
        	      	
        	      	return -1;
        	    }
        	   	else{
                  	GetCtrlVal( panelStart, PANEL_STAR_IPADDRESS, ipString );
					GetCtrlVal( panelStart, PANEL_STAR_IPPORT, ipPortNumber );
					
					Fmt(resource_Name, "%s<TCPIP0::%s::%s::SOCKET", ipString, ipPortNumber );
                }
          	}
			else if( rs232OnLine == 1 ){
			   	port_open = 0;
			   	GetCtrlVal (panelStart, PANEL_STAR_COM1, &port1);
			   	GetCtrlVal (panelStart, PANEL_STAR_COM2, &port2);
			   	GetCtrlVal (panelStart, PANEL_STAR_COM3, &port3);
			   	GetCtrlVal (panelStart, PANEL_STAR_COM4, &port4);
			   	GetCtrlVal (panelStart, PANEL_STAR_BAUDS, &baudRate);
			   	DisableBreakOnLibraryErrors();   
			   	
			   	if( baudRate == 0 ){
			   		baudRate_number = 9600;	
			   	}
			   	else if( baudRate == 1 ){
			   		baudRate_number = 19200;	
			   	}
			   	else if( baudRate == 2 ){
			   		baudRate_number = 38400;	
			   	}
			   	else if( baudRate == 3 ){
			   		baudRate_number = 57600;	
			   	}
			   	else if( baudRate == 4 ){
			   		baudRate_number = 115200;	
			   	}
			   	else if( baudRate == 5 ){
			   		baudRate_number = 230400;	
			   	}
				else if( baudRate == 6 ){
			   		baudRate_number = 460800;	
			   	}
				
			   	if( port1 == 1 ){
			   		comPortNo = 1;
			   		port_open = 1;
			    }
			    else if( port2 == 1 ){
			   		comPortNo = 2;
			   		port_open = 1;
			   	}
			   	else if( port3 == 1 ){
			   		comPortNo = 3;
			   		port_open = 1;
			   	}
			   	else if( port4 == 1 ){
			   		comPortNo = 5;//4;
			   		port_open = 1;
			   	}	
			   		
			   	if( RS232Error == 0 ){
			      	Fmt(resource_Name, "%s<ASRL%i::INSTR", comPortNo); 
			    }
			   	else{
			   	  	DisplayRS232Error(); 
			   	  	return -1;
			   	}  
			}
			
			/* Check for communication */  
			checkErr( ciCs_InitWithOptions (resource_Name,	// resourceName 
											VI_TRUE,   // IdQuery
											VI_TRUE,			// resetDevice 
											"Simulate=0,RangeCheck=1,QueryInstrStatus=1,Cache=1",
											&ciCs, baudRate_number));	
											
			status = SetMouseCursor (VAL_DEFAULT_CURSOR);
			EnableBreakOnLibraryErrors();     
			
			//if( lanOnLine == 1 )
			//	ciCs_WriteInstrData (ciCs, "*IDN?\n"); 
			//else
				ciCs_WriteInstrData (ciCs, "*IDN?"); 
				
			ciCs_ReadInstrData (ciCs, 256, bufData, &CntNumber);

			MessagePopup( "Lx/Ls Series Communication", (char *)bufData );			
			
			if( (panelSteady = LoadPanel(0, "ciCs_example.uir", PANEL_ST)) < 0 )
				return -1;	  
			
			Query_display();
			
			HidePanel( panelStart );			
			DisplayPanel( panelSteady ); 
			
			Error:
		    if (error != VI_SUCCESS)
		        {
		            ViChar   errStr[2048];

		            BuildErrorString (ciCs, error, errStr);
				#ifdef _CVI_
		           	MessagePopup ("Error!", errStr);
				#else
					printf("Message:\n%s", errStr);
				#endif
		        }
	        
			break;
		case EVENT_RIGHT_CLICK:
		    MessagePopup("Cs Series Controller", "Choose the address or com port and baud rate before selecting the interface.");
	}
	return 0;
}

int CVICALLBACK rs232 (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
		    SetCtrlVal(panelStart, PANEL_STAR_GPIB, 0);
		    SetCtrlVal(panelStart, PANEL_STAR_RS232, 1);
			SetCtrlVal(panelStart, PANEL_STAR_LAN, 0);
			SetCtrlAttribute(panelStart, PANEL_STAR_COM1, ATTR_VISIBLE, 1);
			SetCtrlAttribute(panelStart, PANEL_STAR_COM2, ATTR_VISIBLE, 1);
			SetCtrlAttribute(panelStart, PANEL_STAR_COM3, ATTR_VISIBLE, 1);
			SetCtrlAttribute(panelStart, PANEL_STAR_COM4, ATTR_VISIBLE, 1);
			SetCtrlAttribute(panelStart, PANEL_STAR_BAUDS, ATTR_VISIBLE, 1);
			SetCtrlAttribute (panelStart, PANEL_STAR_GPIBADDRESS, ATTR_VISIBLE, 0);
			SetCtrlAttribute(panelStart, PANEL_STAR_RS232SETTING, ATTR_VISIBLE, 1);
			SetCtrlAttribute(panelStart, PANEL_STAR_IEEESETTING, ATTR_VISIBLE, 0);
			
			SetCtrlAttribute(panelStart, PANEL_STAR_LANSETTING, ATTR_VISIBLE, 0);
			SetCtrlAttribute(panelStart, PANEL_STAR_IPADDRESS, ATTR_VISIBLE, 0);
			SetCtrlAttribute(panelStart, PANEL_STAR_IPPORT, ATTR_VISIBLE, 0);
			rs232OnLine = 1;
			break;
		case EVENT_RIGHT_CLICK:
			break;
	}
	return 0;
}

int CVICALLBACK gpib (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
		    SetCtrlVal(panelStart, PANEL_STAR_RS232, 0);
		    SetCtrlVal(panelStart, PANEL_STAR_GPIB, 1);
			SetCtrlVal(panelStart, PANEL_STAR_LAN, 0);
			SetCtrlAttribute(panelStart, PANEL_STAR_COM1, ATTR_VISIBLE, 0);
			SetCtrlAttribute(panelStart, PANEL_STAR_COM2, ATTR_VISIBLE, 0);
			SetCtrlAttribute(panelStart, PANEL_STAR_COM3, ATTR_VISIBLE, 0);
			SetCtrlAttribute(panelStart, PANEL_STAR_COM4, ATTR_VISIBLE, 0);
			SetCtrlAttribute(panelStart, PANEL_STAR_BAUDS, ATTR_VISIBLE, 0);
			SetCtrlAttribute (panelStart, PANEL_STAR_GPIBADDRESS, ATTR_VISIBLE, 1);
			SetCtrlAttribute(panelStart, PANEL_STAR_RS232SETTING, ATTR_VISIBLE, 0);
			SetCtrlAttribute(panelStart, PANEL_STAR_IEEESETTING, ATTR_VISIBLE, 1);
			
			SetCtrlAttribute(panelStart, PANEL_STAR_LANSETTING, ATTR_VISIBLE, 0);
			SetCtrlAttribute(panelStart, PANEL_STAR_IPADDRESS, ATTR_VISIBLE, 0);  
			SetCtrlAttribute(panelStart, PANEL_STAR_IPPORT, ATTR_VISIBLE, 0);
			gpibOnLine = 1;
			break;
		case EVENT_RIGHT_CLICK:
			break;
	}
	return 0;
}

int CVICALLBACK lanSelect (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
		    SetCtrlVal(panelStart, PANEL_STAR_GPIB, 0);
		    SetCtrlVal(panelStart, PANEL_STAR_RS232, 0);
			SetCtrlVal(panelStart, PANEL_STAR_LAN, 1);
			SetCtrlAttribute(panelStart, PANEL_STAR_COM1, ATTR_VISIBLE, 0);
			SetCtrlAttribute(panelStart, PANEL_STAR_COM2, ATTR_VISIBLE, 0);
			SetCtrlAttribute(panelStart, PANEL_STAR_COM3, ATTR_VISIBLE, 0);
			SetCtrlAttribute(panelStart, PANEL_STAR_COM4, ATTR_VISIBLE, 0);
			SetCtrlAttribute(panelStart, PANEL_STAR_BAUDS, ATTR_VISIBLE, 0);
			SetCtrlAttribute (panelStart, PANEL_STAR_GPIBADDRESS, ATTR_VISIBLE, 0);
			SetCtrlAttribute(panelStart, PANEL_STAR_RS232SETTING, ATTR_VISIBLE, 0);
			SetCtrlAttribute(panelStart, PANEL_STAR_IEEESETTING, ATTR_VISIBLE, 0);
																				  
			SetCtrlAttribute(panelStart, PANEL_STAR_LANSETTING, ATTR_VISIBLE, 1);
			SetCtrlAttribute(panelStart, PANEL_STAR_IPADDRESS, ATTR_VISIBLE, 1);
			SetCtrlAttribute(panelStart, PANEL_STAR_IPPORT, ATTR_VISIBLE, 1);
			rs232OnLine = 1;
			break;
		case EVENT_RIGHT_CLICK:
			break;
	}
	return 0;
}

int CVICALLBACK comm1 (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
		    SetCtrlVal(panelStart, PANEL_STAR_COM1, 1);
		    SetCtrlVal(panelStart, PANEL_STAR_COM2, 0);
		    SetCtrlVal(panelStart, PANEL_STAR_COM3, 0);
		    SetCtrlVal(panelStart, PANEL_STAR_COM4, 0);
			break;
		case EVENT_RIGHT_CLICK:
			break;
	}
	return 0;
}

int CVICALLBACK comm2 (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
		    SetCtrlVal(panelStart, PANEL_STAR_COM1, 0);
		    SetCtrlVal(panelStart, PANEL_STAR_COM2, 1);
		    SetCtrlVal(panelStart, PANEL_STAR_COM3, 0);
		    SetCtrlVal(panelStart, PANEL_STAR_COM4, 0);
			break;
		case EVENT_RIGHT_CLICK:
			break;
	}
	return 0;
}

int CVICALLBACK comm3 (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
		    SetCtrlVal(panelStart, PANEL_STAR_COM1, 0);
		    SetCtrlVal(panelStart, PANEL_STAR_COM2, 0);
		    SetCtrlVal(panelStart, PANEL_STAR_COM3, 1);
		    SetCtrlVal(panelStart, PANEL_STAR_COM4, 0);
			break;
		case EVENT_RIGHT_CLICK:
			break;
	}
	return 0;
}

int CVICALLBACK comm4 (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
		    SetCtrlVal(panelStart, PANEL_STAR_COM1, 0);
		    SetCtrlVal(panelStart, PANEL_STAR_COM2, 0);
		    SetCtrlVal(panelStart, PANEL_STAR_COM3, 0);
		    SetCtrlVal(panelStart, PANEL_STAR_COM4, 1);
			break;
		case EVENT_RIGHT_CLICK:
			break;
	}
	return 0;
}

int CVICALLBACK quit (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
		    QuitUserInterface (0);
			break;
		case EVENT_RIGHT_CLICK:
		    
			break;
	}
	return 0;
}		

void DisplayRS232Error (void)
{
    char ErrorMessage[200];
    char *RS232Message;
    
    if (RS232Error < 0) {   /* Bug? ComWrtByte sets rs232error if sent a byte out? */
    
       RS232Message = GetRS232ErrorString (RS232Error); 
       Fmt (ErrorMessage, "%s<RS232 error number  %i : %s", RS232Error, RS232Message);
       MessagePopup ("RS232 Message", ErrorMessage);
       }
   
}

void DisplayGPIBError (void)
{
    char ErrorMessage[200];
    char GPIBMessage[200]; 
    
   	switch (iberr){
		case 0 : 
	    	Fmt (GPIBMessage, "%s<Operating System error."); 
	        break;
	   	case 1 : 
         	Fmt (GPIBMessage, "%s<Function required GPIB board to be CIC.");
         	break;
      	case 2 : 
         	Fmt (GPIBMessage, "%s<No listeners on the GPIB.");
         	break;
      	case 3 : 
         	Fmt (GPIBMessage, "%s<GPIB board not addressed correctly.");
         	break;
      	case 4 : 
         	Fmt (GPIBMessage, "%s<Invalid argument to function call.");
         	break;
      	case 5 : 
         	Fmt (GPIBMessage, "%s<GPIB board not System Controller as required.");
         	break;
      	case 6 : 
         	Fmt (GPIBMessage, "%s<I/O operation aborted (timeout).");
         	break;
      	case 7 : 
         	Fmt (GPIBMessage, "%s<Nonexistent GPIB board.");
         	break;
      	case 8 : 
         	Fmt (GPIBMessage, "%s<DMA error.");
         	break;
      	case 10 : 
         	Fmt (GPIBMessage, "%s<Asynchronous I/O in progress.");
         	break;
      	case 11 : 
         	Fmt (GPIBMessage, "%s<No capability for operation.");
         	break;
      	case 12 : 
         	Fmt (GPIBMessage, "%s<File system error.");
         	break;
      	case 14 : 
         	Fmt (GPIBMessage, "%s<GPIB bus error.");
         	break;
      	case 15 : 
         	Fmt (GPIBMessage, "%s<Serial Poll status byte queue overflow.");
         	break;
      	case 16 : 
         	Fmt (GPIBMessage, "%s<SRQ stuck in ON position.");
         	break;
      	case 20 : 
         	Fmt (GPIBMessage, "%s<Table problem.");
         	break;
      	default: 
         	Fmt (GPIBMessage, "%s<No error.");
       	break;
  	}  
    
    Fmt (ErrorMessage, "%s<GPIB error number  %i : %s ", iberr, GPIBMessage);
    MessagePopup ("GPIB Message", ErrorMessage);
}

void Error_check (char *error_message, int error_val)
{
   char message[50];
   int response;
   
   Fmt (message, "%s<%s. (Error = %d) Continue?", error_message, error_val);
   response = MessagePopup ("Cs Series", error_message);
   exit (-1);
}

/********************************* STEADY STATE CONTROL *********************************/
void Query_display (void){
 	SetCtrlVal(panelSteady, PANEL_ST_steady_message, "Wait for collecting data...");  
	
   	if( gpibOnLine == 1 )  
     	Fmt (int_message, "%s<GPIB, Address %d", address);
   	else if( lanOnLine == 1 )  
		Fmt (int_message, "%s<LAN -  IP Address : %s, IP Port : %s", ipString, ipPortNumber);  		
	else
     	Fmt (int_message, "%s<RS-232, COM port %d", comPortNo);  
		   
   	SetCtrlVal(panelSteady, PANEL_ST_interface_message, int_message);
    
   	/* Get waveform list */
   	ClearListCtrl (panelSteady, PANEL_ST_Waveform_box);  
   
   	ciCs_QueryDefinedWaveforms (ciCs, waveform);
   	
   	/* Extract waveform name */
   	i = 0;
   	n = strlen(waveform);
   	index = 1;    /* remove the first begining " quote */
  	while( n > 0 ){
     	y = Scan(waveform, "%s[i*]>%s[t34]", index, wave);  
      	y = strlen(wave) + 1;
      	index = y + index;
      	n = n - y;
      	
      	if( n > 0 ){
        	if( strcmp(wave, ",") == 0 ){
        	}
        	else{
        		InsertListItem (panelSteady, PANEL_ST_Waveform_box, -1, wave, 0);  
        	}	
      	}
      	
       	i = i++;
       	if( strncmp(wave, waveform_output, strlen(waveform_output)) != 0 )
          	CheckListItem (panelSteady, PANEL_ST_Waveform_box, i, 1);  
  	}
    
    GetNumListItems (panelSteady, PANEL_ST_Waveform_box, &maxitems); 
    
    //Query limits *****************************************************************
   	//if( lanOnLine == 1 )
	//	ciCs_WriteInstrData (ciCs, "FREQ? MIN"); 
	//else
		ciCs_WriteInstrData (ciCs, "FREQ? MIN"); 
		
		ciCs_ReadInstrData (ciCs, 256, bufData, &CntNumber);
		queryMinFreq = atof(bufData);
		
		ciCs_WriteInstrData (ciCs, "*ESR?"); 
		ciCs_ReadInstrData (ciCs, 256, bufData, &CntNumber);
		
	SetCtrlAttribute (panelSteady, PANEL_ST_frequencyslide, ATTR_MIN_VALUE, queryMinFreq);
	
	//if( lanOnLine == 1 )
	//	ciCs_WriteInstrData (ciCs, "FREQ? MAX"); 
	//else
		ciCs_WriteInstrData (ciCs, "FREQ? MAX"); 
		
		ciCs_ReadInstrData (ciCs, 256, bufData, &CntNumber);
		queryMaxFreq = atof(bufData);
		
		ciCs_WriteInstrData (ciCs, "*ESR?"); 
		ciCs_ReadInstrData (ciCs, 256, bufData, &CntNumber);
		
	SetCtrlAttribute (panelSteady, PANEL_ST_frequencyslide, ATTR_MAX_VALUE, queryMaxFreq);
	
	ciCs_QueryMaxCurrentLimit (ciCs, &queryMaxCurrent);
	queryMaxCurrent = queryMaxCurrent*100;
	queryMaxCurrent = TruncateRealNumber (queryMaxCurrent);
	queryMaxCurrent = queryMaxCurrent/100;
	//********************************************************************************
	
    //Query steady status ************************************************************
    GetCtrlVal (panelSteady, PANEL_ST_A_check, &ACheck);
	GetCtrlVal (panelSteady, PANEL_ST_B_check, &BCheck);
	GetCtrlVal (panelSteady, PANEL_ST_C_check, &CCheck);
			
    //if( lanOnLine == 1 )
	//	ciCs_WriteInstrData (ciCs, "FREQ?"); 
	//else
		ciCs_WriteInstrData (ciCs, "FREQ?"); 
	
		ciCs_ReadInstrData (ciCs, 256, bufData, &CntNumber);
		queryFreq = atof(bufData);
		
		ciCs_WriteInstrData (ciCs, "*ESR?"); 
		ciCs_ReadInstrData (ciCs, 256, bufData, &CntNumber);
		
		
	SetCtrlVal(panelSteady, PANEL_ST_frequencyslide, queryFreq); 
	
	//if( lanOnLine == 1 )
	//	ciCs_WriteInstrData (ciCs, "VOLT?"); 
	//else
		ciCs_WriteInstrData (ciCs, "VOLT?"); 
	
		ciCs_ReadInstrData (ciCs, 256, bufData, &CntNumber);
		queryVoltage = atof(bufData);
		
		ciCs_WriteInstrData (ciCs, "*ESR?"); 
		ciCs_ReadInstrData (ciCs, 256, bufData, &CntNumber);
		
	SetCtrlVal(panelSteady, PANEL_ST_voltageslide, queryVoltage); 
	if(ACheck==1){
   		SetCtrlVal(panelSteady, PANEL_ST_A_volt, queryVoltage);  
   	}
   	
   	if(BCheck==1){
   		SetCtrlVal(panelSteady, PANEL_ST_B_volt, queryVoltage);  
   	}
   	
   	if(CCheck==1){
   		SetCtrlVal(panelSteady, PANEL_ST_C_volt, queryVoltage);  
   	}
	
	//if( lanOnLine == 1 )
	//	ciCs_WriteInstrData (ciCs, "CURR?"); 
	//else
		ciCs_WriteInstrData (ciCs, "CURR?"); 
		
		ciCs_ReadInstrData (ciCs, 256, bufData, &CntNumber);
		queryCurrent = atof(bufData);
		
		ciCs_WriteInstrData (ciCs, "*ESR?"); 
		ciCs_ReadInstrData (ciCs, 256, bufData, &CntNumber);
		
	SetCtrlVal(panelSteady, PANEL_ST_currentslide, queryCurrent); 
	if(ACheck==1){
   		SetCtrlVal(panelSteady, PANEL_ST_A_current, queryCurrent);  
   	}
   	
   	if(BCheck==1){
   		SetCtrlVal(panelSteady, PANEL_ST_B_current, queryCurrent);  
   	}
   	
   	if(CCheck==1){
   		SetCtrlVal(panelSteady, PANEL_ST_C_current, queryCurrent);  
   	}
   	
   	//if( lanOnLine == 1 )
	//	ciCs_WriteInstrData (ciCs, "OUTPUT?"); 
	//else
		ciCs_WriteInstrData (ciCs, "OUTPUT?"); 
		
		ciCs_ReadInstrData (ciCs, 256, bufData, &CntNumber);
		queryOutputState = atof(bufData);
		
		ciCs_WriteInstrData (ciCs, "*ESR?"); 
		ciCs_ReadInstrData (ciCs, 256, bufData, &CntNumber);
		
	if( queryOutputState == 1 )
  	 	SetCtrlVal(panelSteady, PANEL_ST_outputswitch, 1);
   	else
	 	SetCtrlVal(panelSteady, PANEL_ST_outputswitch, 0); 
	  
   	//if( lanOnLine == 1 )
	//	ciCs_WriteInstrData (ciCs, "VOLT:RANG?"); 
	//else
		ciCs_WriteInstrData (ciCs, "VOLT:RANG?"); 
		
		ciCs_ReadInstrData (ciCs, 256, bufData, &CntNumber);
		queryVoltRange = atof(bufData);
		
		ciCs_WriteInstrData (ciCs, "*ESR?"); 
		ciCs_ReadInstrData (ciCs, 256, bufData, &CntNumber);
	
   	if( queryVoltRange == 135.0 ){//< voltage_limit ){
		SetCtrlAttribute (panelSteady, PANEL_ST_rangeswitch1, ATTR_VISIBLE, 1);
		SetCtrlAttribute (panelSteady, PANEL_ST_rangeswitch2, ATTR_VISIBLE, 0);
      	SetCtrlVal(panelSteady, PANEL_ST_rangeswitch1, 0);
	  	SetCtrlAttribute(panelSteady, PANEL_ST_currentslide, ATTR_MAX_VALUE, queryMaxCurrent); 
	  	SetCtrlAttribute(panelSteady, PANEL_ST_voltageslide, ATTR_MAX_VALUE, queryVoltRange);
	}
   	else if( queryVoltRange == 270.0 ){
      	SetCtrlAttribute (panelSteady, PANEL_ST_rangeswitch1, ATTR_VISIBLE, 1);
		SetCtrlAttribute (panelSteady, PANEL_ST_rangeswitch2, ATTR_VISIBLE, 0);
      	SetCtrlVal(panelSteady, PANEL_ST_rangeswitch1, 1); 
	  	SetCtrlAttribute(panelSteady, PANEL_ST_currentslide, ATTR_MAX_VALUE, queryMaxCurrent);
	  	SetCtrlAttribute(panelSteady, PANEL_ST_voltageslide, ATTR_MAX_VALUE, queryVoltRange);
	}
	else if( queryVoltRange == 150.0 ){
      	SetCtrlAttribute (panelSteady, PANEL_ST_rangeswitch1, ATTR_VISIBLE, 0);
		SetCtrlAttribute (panelSteady, PANEL_ST_rangeswitch2, ATTR_VISIBLE, 1);
      	SetCtrlVal(panelSteady, PANEL_ST_rangeswitch2, 0); 
	  	SetCtrlAttribute(panelSteady, PANEL_ST_currentslide, ATTR_MAX_VALUE, queryMaxCurrent);
	  	SetCtrlAttribute(panelSteady, PANEL_ST_voltageslide, ATTR_MAX_VALUE, queryVoltRange);
	}
	else if( queryVoltRange == 300.0 ){
      	SetCtrlAttribute (panelSteady, PANEL_ST_rangeswitch1, ATTR_VISIBLE, 0);
		SetCtrlAttribute (panelSteady, PANEL_ST_rangeswitch2, ATTR_VISIBLE, 1);
      	SetCtrlVal(panelSteady, PANEL_ST_rangeswitch2, 1); 
	  	SetCtrlAttribute(panelSteady, PANEL_ST_currentslide, ATTR_MAX_VALUE, queryMaxCurrent);
	  	SetCtrlAttribute(panelSteady, PANEL_ST_voltageslide, ATTR_MAX_VALUE, queryVoltRange);
	}
	
   	/* Check Phase Mode */
   	//if( lanOnLine == 1 )
	//	ciCs_WriteInstrData (ciCs, "SYST:CONF:NOUT?"); 
	//else
		ciCs_WriteInstrData (ciCs, "SYST:CONF:NOUT?"); 
	
	ciCs_ReadInstrData (ciCs, 256, bufData, &CntNumber);
	queryPhaseMode = atoi(bufData);
	
	if( queryPhaseMode == 1 ){
		SetCtrlVal(panelSteady, PANEL_ST_modeswitch, 1);
		SetCtrlVal(panelSteady, PANEL_ST_B_check, 0);
		SetCtrlVal(panelSteady, PANEL_ST_C_check, 0);
		SetCtrlAttribute(panelSteady, PANEL_ST_B_check, ATTR_DIMMED, 1);
		SetCtrlAttribute(panelSteady, PANEL_ST_C_check, ATTR_DIMMED, 1);
	}
	else{
		SetCtrlVal(panelSteady, PANEL_ST_modeswitch, 0);
		SetCtrlVal(panelSteady, PANEL_ST_B_check, 1);
		SetCtrlVal(panelSteady, PANEL_ST_C_check, 1);
		SetCtrlAttribute(panelSteady, PANEL_ST_B_check, ATTR_DIMMED, 0);
		SetCtrlAttribute(panelSteady, PANEL_ST_C_check, ATTR_DIMMED, 0);
		
   		//if( lanOnLine == 1 )  
		//	ciCs_WriteInstrData (ciCs, ":INST:NSEL 1;:PHAS?"); 
		//else
			ciCs_WriteInstrData (ciCs, ":INST:NSEL 1;:PHAS?"); 
		
		ciCs_ReadInstrData (ciCs, 256, bufData, &CntNumber);
		queryPhaseAngle = atof(bufData);
   		SetCtrlVal(panelSteady, PANEL_ST_A_phase, queryPhaseAngle);  
   		
   		//if( lanOnLine == 1 )
		//	ciCs_WriteInstrData (ciCs, ":INST:NSEL 2;:PHAS?"); 
		//else
			ciCs_WriteInstrData (ciCs, ":INST:NSEL 2;:PHAS?"); 
			
		ciCs_ReadInstrData (ciCs, 256, bufData, &CntNumber);
		queryPhaseAngle = atof(bufData);
   		SetCtrlVal(panelSteady, PANEL_ST_B_phase, queryPhaseAngle);  
   	
		//if( lanOnLine == 1 )
   		//	ciCs_WriteInstrData (ciCs, ":INST:NSEL 3;:PHAS?"); 
		//else
			ciCs_WriteInstrData (ciCs, ":INST:NSEL 3;:PHAS?"); 
		
		ciCs_ReadInstrData (ciCs, 256, bufData, &CntNumber);
		queryPhaseAngle = atof(bufData);
   		SetCtrlVal(panelSteady, PANEL_ST_C_phase, queryPhaseAngle);  
	}
	//********************************************************************************
	
	DisplayPanel (panelSteady);
	SetCtrlVal(panelSteady, PANEL_ST_steady_message, "Ready...");
}

int CVICALLBACK Steady_panel (int panel, int event, void *callbackData,
		int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_GOT_FOCUS:
			Query_display();
			break;
		case EVENT_LOST_FOCUS:
			break;
		case EVENT_CLOSE:
	        status = ConfirmPopup( "Cs Series Controller", "Do you want to close the program?" );
	     	if( status ){
		      	ciCs_close (ciCs);
		      	DiscardPanel (panelSteady);
			   	DisplayPanel (panelStart);
		   	}
			break;
	}
	return 0;
}

int CVICALLBACK close (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
			status = ConfirmPopup ("Cs Series Controller", "Do you want to close the program?");
			if( status ){ 
			   	ciCs_close (ciCs);
			   	
			   	DiscardPanel (panelSteady);
			   	DisplayPanel (panelStart);
			}
			break;
		case EVENT_RIGHT_CLICK:
		    MessagePopup ("Cs Series Controller", "Close the steady state output.");
	}
	return 0;
}

int CVICALLBACK frequency_control (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
			GetCtrlVal (panelSteady, PANEL_ST_frequencyslide, &freq_value);
				
			ciCs_ConfigureFrequencyValue ( ciCs,  freq_value);										  
			break;
	
		case EVENT_RIGHT_CLICK:
		    MessagePopup ("Cs Series Controller", "Change the frequency.");
	}
	return 0;
}

int CVICALLBACK current_control (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
			GetCtrlVal (panelSteady, PANEL_ST_currentslide, &curr_value);
			
			GetCtrlVal (panelSteady, PANEL_ST_A_check, &ACheck);
			GetCtrlVal (panelSteady, PANEL_ST_B_check, &BCheck);
			GetCtrlVal (panelSteady, PANEL_ST_C_check, &CCheck);
			
			if( ACheck==1 && BCheck==1 && CCheck==1 )
			{
				error_status =ciCs_ConfigureCurrentLevel (ciCs, "ALLPHASES", curr_value);
		       	SetCtrlVal(panelSteady, PANEL_ST_A_current, curr_value); 
				SetCtrlVal(panelSteady, PANEL_ST_B_current, curr_value);
				SetCtrlVal(panelSteady, PANEL_ST_C_current, curr_value); 
			}
			else
			{
				if(ACheck==1){
		       		error_status =ciCs_ConfigureCurrentLevel (ciCs, "PHASE1", curr_value);
		       		SetCtrlVal(panelSteady, PANEL_ST_A_current, curr_value);  
		       	}
	       	
		       	if(BCheck==1){
		       		error_status =ciCs_ConfigureCurrentLevel (ciCs, "PHASE2", curr_value);
		       		SetCtrlVal(panelSteady, PANEL_ST_B_current, curr_value);  
		       	}
	       	
		       	if(CCheck==1){
		       		error_status =ciCs_ConfigureCurrentLevel (ciCs, "PHASE3", curr_value);
		       		SetCtrlVal(panelSteady, PANEL_ST_C_current, curr_value);  
		       	}
			
				if( error_status != 0 ){
				   	MessagePopup ("Cs Series Error", "Error programming instrument.\nPlease check your instrument.");
				   	return -1;
				}
			}
			break;
	
		case EVENT_RIGHT_CLICK:
		    MessagePopup ("Cs Series Controller", "Change the current limit.");
	}
	return 0;
}

int CVICALLBACK voltage_control (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
			GetCtrlVal (panelSteady, PANEL_ST_voltageslide, &volt_value);
			GetCtrlVal (panelSteady, PANEL_ST_A_check, &ACheck);
			GetCtrlVal (panelSteady, PANEL_ST_B_check, &BCheck);
			GetCtrlVal (panelSteady, PANEL_ST_C_check, &CCheck);
			
			if( ACheck==1 && BCheck==1 && CCheck==1 )
			{
				error_status =ciCs_ConfigureVoltageLimit (ciCs, "ALLPHASES", CICS_VAL_VOLTAGE_REGULATE, volt_value);
		       	SetCtrlVal(panelSteady, PANEL_ST_A_volt, volt_value); 
				SetCtrlVal(panelSteady, PANEL_ST_B_volt, volt_value);
				SetCtrlVal(panelSteady, PANEL_ST_C_volt, volt_value); 
			}
			else
			{
		       	if(ACheck==1){
		       		error_status =ciCs_ConfigureVoltageLimit (ciCs, "PHASE1", CICS_VAL_VOLTAGE_REGULATE, volt_value);
		       		SetCtrlVal(panelSteady, PANEL_ST_A_volt, volt_value);  
		       	}
	       	
		       	if(BCheck==1){
		       		error_status =ciCs_ConfigureVoltageLimit (ciCs, "PHASE2", CICS_VAL_VOLTAGE_REGULATE, volt_value);
		       		SetCtrlVal(panelSteady, PANEL_ST_B_volt, volt_value);  
		       	}
	       	
		       	if(CCheck==1){
		       		error_status =ciCs_ConfigureVoltageLimit (ciCs, "PHASE3", CICS_VAL_VOLTAGE_REGULATE, volt_value);
		       		SetCtrlVal(panelSteady, PANEL_ST_C_volt, volt_value);  
		       	}
			}
	       	
	       	SetCtrlAttribute(panelSteady, PANEL_ST_A_phase, ATTR_DIMMED, 0);
   	       	SetCtrlAttribute(panelSteady, PANEL_ST_phaseslide, ATTR_DIMMED, 0); 
   	       	SetCtrlAttribute(panelSteady, PANEL_ST_frequencyslide, ATTR_DIMMED, 0);
   	       	SetCtrlAttribute(panelSteady, PANEL_ST_Waveform_box, ATTR_DIMMED, 0); 
           	SetCtrlAttribute(panelSteady, PANEL_ST_THD, ATTR_DIMMED, 0); 
           	SetCtrlAttribute(panelSteady, PANEL_ST_Clip_Level, ATTR_DIMMED, 0);
	   	  
			if( error_status != 0 ){
			   	MessagePopup ("Cs Series Error", "Error programming instrument.\nPlease check your instrument.");
			   	return -1;
			}		
		
			break;
	
		case EVENT_RIGHT_CLICK:
		    MessagePopup ("Cs Series Controller", "Change the voltage.");
	}
	return 0;
}

int CVICALLBACK register_control (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
			GetCtrlVal (panelSteady, PANEL_ST_register_number, &register_number);
			GetCtrlVal (panelSteady, PANEL_ST_register_slide, &register_status);
			if( register_status == 0 ){
			   	error_status = ciCs_StoreRecallRegister(ciCs, 0, register_number);
			    if( error_status != 0 ){
			       	MessagePopup ("Cs Series Error", "Error programming instrument.\nPlease check your instrument.");
			       	return -1;
			    }
			}
			else{
			   	error_status = ciCs_StoreRecallRegister(ciCs, 1, register_number);  
			   	if( error_status != 0 ){
			       	MessagePopup ("Cs Series Error", "Error programming instrument.\nPlease check your instrument.");
			       	return -1;
			   	}
			}
			Query_display(); 
			break;
		case EVENT_RIGHT_CLICK:
		    MessagePopup ("Cs Series Controller", "Recall or save the register.");
	}
	return 0;
}

int CVICALLBACK sourceALC (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	int select, error_status=0;
	switch (event) {
		case EVENT_COMMIT:
			GetCtrlVal (panelSteady, PANEL_ST_SOURCEALC, &select);
			if( select == 0 ){
			    error_status =ciCs_ConfigureOutputALCState (ciCs, 0);
			    if( error_status != 0 ){
			       	MessagePopup ("Cs Series Error", "Error programming Source ALC. \nPlease check your instrument.");
			       	return -1;
			   	}
			}
			else if( select == 1 ){ 
			    error_status =ciCs_ConfigureOutputALCState (ciCs, 1);
			    if( error_status != 0 ){
			       	MessagePopup ("Cs Series Error", "Error programming Source ALC. \nPlease check your instrument.");
			       	return -1;
			   	}
			}
			else if( select == 2 ){ 
			    error_status =ciCs_ConfigureOutputALCState (ciCs, 2);
			    if( error_status != 0 ){
			       	MessagePopup ("Cs Series Error", "Error programming Source ALC. \nPlease check your instrument.");
			       	return -1;
			   	}
			}
			break;
		case EVENT_RIGHT_CLICK:
		    MessagePopup ("Cs Series Controller", "Enable or Disable Source ALC.");
	}
	return 0;
}

int CVICALLBACK Output_relay (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
			GetCtrlVal (panelSteady, PANEL_ST_outputswitch, &relay);
			if( relay == 1 ){
			    error_status =ciCs_ConfigureOutputEnabled (ciCs, VI_TRUE);
			    if( error_status != 0 ){
			       	MessagePopup ("Cs Series Error", "Error programming instrument.\nPlease check your instrument.");
			       	return -1;
			   	}
			}
			else{
			    error_status =ciCs_ConfigureOutputEnabled (ciCs, VI_FALSE);//);
			    if( error_status != 0 ){
			       	MessagePopup ("Cs Series Error", "Error programming instrument.\nPlease check your instrument.");
			       	return -1;
			   	}
			}
			
			break;
		case EVENT_RIGHT_CLICK:
		    MessagePopup ("Cs Series Controller", "Close or open the relay output.");
	}
	return 0;
}

int CVICALLBACK range_control1 (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
			GetCtrlVal( panelSteady, PANEL_ST_rangeswitch1, &range_button );
			if( range_button == 0 ){
			   	error_status = ciCs_ConfigureOutputRange (ciCs, 135.0);
			   	if( error_status != 0 ){
			      	MessagePopup ("Cs Series Error", "Error programming instrument.\nPlease check your instrument.");
			      	return -1;
			   	}
			}	
			else if( range_button == 1 ){
			    error_status = ciCs_ConfigureOutputRange (ciCs, 270.0);
			    if( error_status != 0 ){
			       	MessagePopup ("Cs Series Error", "Error programming instrument.\nPlease check your instrument.");
			       	return -1;
			   	}
			}
			
			Query_display(); 
			break;
		case EVENT_RIGHT_CLICK:
		    MessagePopup ("Cs Series Controller", "Change the voltage range.");
	}
	return 0;
}

int CVICALLBACK range_control2 (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
			GetCtrlVal( panelSteady, PANEL_ST_rangeswitch2, &range_button );
			if( range_button == 0 ){
				error_status = ciCs_ConfigureOutputRange (ciCs, 150.0);
			   	if( error_status != 0 ){
			      	MessagePopup ("Cs Series Error", "Error programming instrument.\nPlease check your instrument.");
			      	return -1;
			   	}
			}	
			else if( range_button == 1 ){
			    error_status = ciCs_ConfigureOutputRange (ciCs, 300.0);
			    if( error_status != 0 ){
			       	MessagePopup ("Cs Series Error", "Error programming instrument.\nPlease check your instrument.");
			       	return -1;
			   	}
			}
		
			Query_display(); 
			break;
		case EVENT_RIGHT_CLICK:
		    MessagePopup ("Cs Series Controller", "Change the voltage range.");
	}
	return 0;
}

int CVICALLBACK Mode_control (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch( event ){
		case EVENT_COMMIT:
			GetCtrlVal( panelSteady, PANEL_ST_modeswitch, &mode_option );
			if( mode_option == 1 ){
			   	error_status =ciCs_ConfigureOutputPhaseMode (ciCs, CICS_VAL_1_PHASE);
			   	if( error_status != 0 ){
			      	MessagePopup ("Cs Series Error", "Error programming instrument.\nPlease check your instrument.");
			      	return -1;
			    }
			}	
			else{
			    error_status =ciCs_ConfigureOutputPhaseMode (ciCs, CICS_VAL_3_PHASE);
			    if( error_status != 0 ){
			       	MessagePopup ("Cs Series Error", "Error programming instrument.\nPlease check your instrument.");
			       	return -1;
			   	}
			}
			Query_display();
			break;
		case EVENT_RIGHT_CLICK:
		    MessagePopup ("Cs Series Controller", "Change phase mode to single or three phase.");
		
	}
	return 0;
}

int CVICALLBACK phase_control (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
			GetCtrlVal(panelSteady, PANEL_ST_phaseslide, &phase_angle);
			
			GetCtrlVal (panelSteady, PANEL_ST_A_check, &ACheck);
			GetCtrlVal (panelSteady, PANEL_ST_B_check, &BCheck);
			GetCtrlVal (panelSteady, PANEL_ST_C_check, &CCheck);
			
	       	if(ACheck==1){
	       		error_status =ciCs_ConfigureOutput (ciCs, CICS_VAL_3_PHASE); 
			    error_status =ciCs_ConfigureOutputPhaseMode (ciCs, CICS_VAL_3_PHASE);
			    error_status =ciCs_ConfigurePhaseAngle (ciCs, "PHASE1", 0);
	       		SetCtrlVal(panelSteady, PANEL_ST_A_phase, 0.00);  
	       	}
	       	
	       	if(BCheck==1){
	       		error_status =ciCs_ConfigureCurrentLevel (ciCs, "PHASE2", curr_value);
	       		error_status =ciCs_ConfigurePhaseAngle (ciCs, "PHASE2", phase_angle);
	       		SetCtrlVal(panelSteady, PANEL_ST_B_phase, phase_angle);  
	       	}
	       	
	       	if(CCheck==1){
	       		error_status =ciCs_ConfigureCurrentLevel (ciCs, "PHASE3", curr_value);
	       		error_status =ciCs_ConfigurePhaseAngle (ciCs, "PHASE3", phase_angle);
	       		SetCtrlVal(panelSteady, PANEL_ST_C_phase, phase_angle);  
	       	}
	       	
			break;
		case EVENT_RIGHT_CLICK:
		    MessagePopup ("Cs Series Controller", "Change phase angle.");
		
	}
	return 0;
}

int CVICALLBACK waveform_control (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	char label[12];
	ViReal64 freq;
	
	switch (event) {
		case EVENT_COMMIT:
			GetNumCheckedItems (panelSteady, PANEL_ST_Waveform_box, &numitems); 
			if( numitems > 1 ){
			   	MessagePopup ("Cs Series Controller", "Choose one waveform only.");   
			   	break;
			}
			GetNumListItems (panelSteady, PANEL_ST_Waveform_box, &maxitems); 
			for( i = 0; i < maxitems; i++ ){
			   	IsListItemChecked (panelSteady, PANEL_ST_Waveform_box, i, &checked);
			   	if( checked ){
			      	GetLabelFromIndex (panelSteady, PANEL_ST_Waveform_box, i, label); 
			      	GetCtrlVal(panelSteady, PANEL_ST_THD, &clip_level);
			      	GetCtrlVal(panelSteady, PANEL_ST_frequencyslide, &freq);
					
					ciCs_ConfigureWaveform (ciCs, label, freq);
					ciCs_ConfigureClippingLevel (ciCs, clip_level);
			   	}
			}
			break;
		case EVENT_RIGHT_CLICK:
		    MessagePopup ("Cs Series Controller", "Clipping waveform.");
		
	}
	return 0;
}

/********************************** MEASUREMENT CONTROL **********************************/
void CVICALLBACK Measure_panel (int menuBar, int menuItem, void *callbackData,
	                        	int panel)
{
	panelMeas = LoadPanel (0, "ciCs_example.uir", PANEL_MEAS);
	DisplayPanel (panelMeas); 
	
	if( queryPhaseMode == 1 ){ 
	   	SetCtrlAttribute(panelMeas, PANEL_MEAS_B_Check, ATTR_DIMMED, 1); 
	   	SetCtrlAttribute(panelMeas, PANEL_MEAS_B_volt, ATTR_DIMMED, 1);
	   	SetCtrlAttribute(panelMeas, PANEL_MEAS_B_current, ATTR_DIMMED, 1);
	   	SetCtrlAttribute(panelMeas, PANEL_MEAS_B_phase, ATTR_DIMMED, 1);
	   	SetCtrlAttribute(panelMeas, PANEL_MEAS_B_power, ATTR_DIMMED, 1); 
	   	SetCtrlAttribute(panelMeas, PANEL_MEAS_B_Pfactor, ATTR_DIMMED, 1); 
	   	SetCtrlAttribute(panelMeas, PANEL_MEAS_B_app_pow, ATTR_DIMMED, 1); 
	   	SetCtrlAttribute(panelMeas, PANEL_MEAS_B_peak_curr, ATTR_DIMMED, 1);
	   	SetCtrlAttribute(panelMeas, PANEL_MEAS_B_crest_fact, ATTR_DIMMED, 1); 
	   	
	   	SetCtrlAttribute(panelMeas, PANEL_MEAS_C_Check, ATTR_DIMMED, 1);
	   	SetCtrlAttribute(panelMeas, PANEL_MEAS_C_volt, ATTR_DIMMED, 1);
	   	SetCtrlAttribute(panelMeas, PANEL_MEAS_C_current, ATTR_DIMMED, 1);
	   	SetCtrlAttribute(panelMeas, PANEL_MEAS_C_phase, ATTR_DIMMED, 1);
	   	SetCtrlAttribute(panelMeas, PANEL_MEAS_C_power, ATTR_DIMMED, 1);
	   	SetCtrlAttribute(panelMeas, PANEL_MEAS_C_Pfactor, ATTR_DIMMED, 1);
	   	SetCtrlAttribute(panelMeas, PANEL_MEAS_C_app_pow, ATTR_DIMMED, 1);
	   	SetCtrlAttribute(panelMeas, PANEL_MEAS_C_peak_curr, ATTR_DIMMED, 1);
	   	SetCtrlAttribute(panelMeas, PANEL_MEAS_C_crest_fact, ATTR_DIMMED, 1); 
	}
}

int CVICALLBACK measure (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
			SetCtrlAttribute(panelMeas, PANEL_MEAS_meas_button, ATTR_DIMMED, 1);
		    
		    if( queryPhaseMode == 1 ){ 
			   	SetCtrlAttribute(panelMeas, PANEL_MEAS_B_Check, ATTR_DIMMED, 1); 
			   	SetCtrlAttribute(panelMeas, PANEL_MEAS_B_volt, ATTR_DIMMED, 1);
			   	SetCtrlAttribute(panelMeas, PANEL_MEAS_B_current, ATTR_DIMMED, 1);
			   	SetCtrlAttribute(panelMeas, PANEL_MEAS_B_phase, ATTR_DIMMED, 1);
			   	SetCtrlAttribute(panelMeas, PANEL_MEAS_B_power, ATTR_DIMMED, 1); 
			   	SetCtrlAttribute(panelMeas, PANEL_MEAS_B_Pfactor, ATTR_DIMMED, 1); 
			   	SetCtrlAttribute(panelMeas, PANEL_MEAS_B_app_pow, ATTR_DIMMED, 1); 
			   	SetCtrlAttribute(panelMeas, PANEL_MEAS_B_peak_curr, ATTR_DIMMED, 1);
			   	SetCtrlAttribute(panelMeas, PANEL_MEAS_B_crest_fact, ATTR_DIMMED, 1); 
	   	
			   	SetCtrlAttribute(panelMeas, PANEL_MEAS_C_Check, ATTR_DIMMED, 1);
			   	SetCtrlAttribute(panelMeas, PANEL_MEAS_C_volt, ATTR_DIMMED, 1);
			   	SetCtrlAttribute(panelMeas, PANEL_MEAS_C_current, ATTR_DIMMED, 1);
			   	SetCtrlAttribute(panelMeas, PANEL_MEAS_C_phase, ATTR_DIMMED, 1);
			   	SetCtrlAttribute(panelMeas, PANEL_MEAS_C_power, ATTR_DIMMED, 1);
			   	SetCtrlAttribute(panelMeas, PANEL_MEAS_C_Pfactor, ATTR_DIMMED, 1);
			   	SetCtrlAttribute(panelMeas, PANEL_MEAS_C_app_pow, ATTR_DIMMED, 1);
			   	SetCtrlAttribute(panelMeas, PANEL_MEAS_C_peak_curr, ATTR_DIMMED, 1);
			   	SetCtrlAttribute(panelMeas, PANEL_MEAS_C_crest_fact, ATTR_DIMMED, 1); 
			}
			else{
				SetCtrlAttribute(panelMeas, PANEL_MEAS_B_Check, ATTR_DIMMED, 0); 
			   	SetCtrlAttribute(panelMeas, PANEL_MEAS_B_volt, ATTR_DIMMED, 0);
			   	SetCtrlAttribute(panelMeas, PANEL_MEAS_B_current, ATTR_DIMMED, 0);
			   	SetCtrlAttribute(panelMeas, PANEL_MEAS_B_phase, ATTR_DIMMED, 0);
			   	SetCtrlAttribute(panelMeas, PANEL_MEAS_B_power, ATTR_DIMMED, 0); 
			   	SetCtrlAttribute(panelMeas, PANEL_MEAS_B_Pfactor, ATTR_DIMMED, 0); 
			   	SetCtrlAttribute(panelMeas, PANEL_MEAS_B_app_pow, ATTR_DIMMED, 0); 
			   	SetCtrlAttribute(panelMeas, PANEL_MEAS_B_peak_curr, ATTR_DIMMED, 0);
			   	SetCtrlAttribute(panelMeas, PANEL_MEAS_B_crest_fact, ATTR_DIMMED, 0); 
	   	
			   	SetCtrlAttribute(panelMeas, PANEL_MEAS_C_Check, ATTR_DIMMED, 0);
			   	SetCtrlAttribute(panelMeas, PANEL_MEAS_C_volt, ATTR_DIMMED, 0);
			   	SetCtrlAttribute(panelMeas, PANEL_MEAS_C_current, ATTR_DIMMED, 0);
			   	SetCtrlAttribute(panelMeas, PANEL_MEAS_C_phase, ATTR_DIMMED, 0);
			   	SetCtrlAttribute(panelMeas, PANEL_MEAS_C_power, ATTR_DIMMED, 0);
			   	SetCtrlAttribute(panelMeas, PANEL_MEAS_C_Pfactor, ATTR_DIMMED, 0);
			   	SetCtrlAttribute(panelMeas, PANEL_MEAS_C_app_pow, ATTR_DIMMED, 0);
			   	SetCtrlAttribute(panelMeas, PANEL_MEAS_C_peak_curr, ATTR_DIMMED, 0);
			   	SetCtrlAttribute(panelMeas, PANEL_MEAS_C_crest_fact, ATTR_DIMMED, 0);
			}
	
		    GetCtrlVal (panelMeas, PANEL_MEAS_A_Check, &checkA); 
		    GetCtrlVal (panelMeas, PANEL_MEAS_B_Check, &checkB); 
		    GetCtrlVal (panelMeas, PANEL_MEAS_C_Check, &checkC); 
		    GetCtrlVal (panelMeas, PANEL_MEAS_Volt_Check, &meas_volt);
		    GetCtrlVal (panelMeas, PANEL_MEAS_Phase_Check, &meas_current); 
		    GetCtrlVal (panelMeas, PANEL_MEAS_Current_Check, &meas_phase); 
		    GetCtrlVal (panelMeas, PANEL_MEAS_Power_Check, &meas_power); 
		    GetCtrlVal (panelMeas, PANEL_MEAS_App_Pow_Check, &meas_app_pow);
		    GetCtrlVal (panelMeas, PANEL_MEAS_PFactor_Check, &meas_pfactor);
		    GetCtrlVal (panelMeas, PANEL_MEAS_Peak_Curr_Check, &meas_peak_curr);  
		    GetCtrlVal (panelMeas, PANEL_MEAS_Crest_Fact_Check, &meas_crest_fact);  
		    
			ciCs_Measure (ciCs, "PHASE1", 10000, CICS_VAL_MEASURE_FREQUENCY, &frequency);
			SetCtrlVal(panelMeas, PANEL_MEAS_Frequency, frequency);
			
			if( meas_volt == 1 ){
			   	if( checkA == 1 ){
			      	ciCs_Measure (ciCs, "PHASE1", 10000, CICS_VAL_MEASURE_AC_VOLTAGE_RMS, &voltage); 
			      	SetCtrlVal(panelMeas, PANEL_MEAS_A_volt, voltage);
			    }
			   	if( checkB == 1 ){
			      	ciCs_Measure (ciCs, "PHASE2", 10000, CICS_VAL_MEASURE_AC_VOLTAGE_RMS, &voltage); 
			      	SetCtrlVal(panelMeas, PANEL_MEAS_B_volt, voltage);
			    }
			   	if( checkC == 1 ){
			      	ciCs_Measure (ciCs, "PHASE3", 10000, CICS_VAL_MEASURE_AC_VOLTAGE_RMS, &voltage); 
			      	SetCtrlVal(panelMeas, PANEL_MEAS_C_volt, voltage);
			    }
			}
			if( meas_current == 1 ){
			   	if( checkA == 1 ){
					ciCs_Measure (ciCs, "PHASE1", 10000, CICS_VAL_MEASURE_AC_CURRENT_RMS, &current);
			      	SetCtrlVal(panelMeas, PANEL_MEAS_A_current, current);
			   	}
			   	if( checkB == 1 ){
			      	ciCs_Measure (ciCs, "PHASE2", 10000, CICS_VAL_MEASURE_AC_CURRENT_RMS, &current);
			      	SetCtrlVal(panelMeas, PANEL_MEAS_B_current, current);
			     }
			   	if( checkC == 1 ){
			      	ciCs_Measure (ciCs, "PHASE3", 10000, CICS_VAL_MEASURE_AC_CURRENT_RMS, &current);
			      	SetCtrlVal(panelMeas, PANEL_MEAS_C_current, current);
			    }
			}
			
			if( meas_phase == 1 ){
			   	if( checkA == 1 ){
			      	SetCtrlVal(panelMeas, PANEL_MEAS_A_phase, 0.0);
			    }
			   	if( checkB == 1 ){
					ciCs_Measure (ciCs, "PHASE2", 10000, CICS_VAL_MEASURE_PHASE, &phaseangle);
			      	SetCtrlVal(panelMeas, PANEL_MEAS_B_phase, phaseangle);
			   	}
			   	if( checkC == 1 ){
			      	ciCs_Measure (ciCs, "PHASE3", 10000, CICS_VAL_MEASURE_PHASE, &phaseangle);
			      	SetCtrlVal(panelMeas, PANEL_MEAS_C_phase, phaseangle);
			    }
			}
			
			if( meas_power == 1 ){
			   	if( checkA == 1 ){
					ciCs_Measure (ciCs, "PHASE1", 10000, CICS_VAL_MEASURE_AC_POWER, &power);
			      	SetCtrlVal(panelMeas, PANEL_MEAS_A_power, power);
			   	}
			   	if( checkB == 1 ){
			      	ciCs_Measure (ciCs, "PHASE2", 10000, CICS_VAL_MEASURE_AC_POWER, &power);
			      	SetCtrlVal(panelMeas, PANEL_MEAS_B_power, power);
			   	}
			   	if( checkC == 1 ){
			      	ciCs_Measure (ciCs, "PHASE3", 10000, CICS_VAL_MEASURE_AC_POWER, &power);
			      	SetCtrlVal(panelMeas, PANEL_MEAS_C_power, power);
			   	}
			}
			
			if(	meas_pfactor == 1 ){
			   	if( checkA == 1 ){
					ciCs_Measure (ciCs, "PHASE1", 10000, CICS_VAL_MEASURE_AC_POWER_FACTOR, &powerfactor);
			      	SetCtrlVal(panelMeas, PANEL_MEAS_A_Pfactor, powerfactor);
			   	}
			   	if( checkB == 1 ){
			      	ciCs_Measure (ciCs, "PHASE2", 10000, CICS_VAL_MEASURE_AC_POWER_FACTOR, &powerfactor);
			      	SetCtrlVal(panelMeas, PANEL_MEAS_B_Pfactor, powerfactor);
			   	}
			   	if( checkC == 1 ){
			      	ciCs_Measure (ciCs, "PHASE3", 10000, CICS_VAL_MEASURE_AC_POWER_FACTOR, &powerfactor);
			      	SetCtrlVal(panelMeas, PANEL_MEAS_C_Pfactor, powerfactor);
			    }
			}
			 
			if( meas_app_pow == 1 ){
			   	if( checkA == 1 ){
					ciCs_Measure (ciCs, "PHASE1", 10000, CICS_VAL_MEASURE_AC_APPARENT_POWER, &app_pow);
			      	SetCtrlVal(panelMeas, PANEL_MEAS_A_app_pow, app_pow);
			  	}
			   	if( checkB == 1 ){
			      	ciCs_Measure (ciCs, "PHASE2", 10000, CICS_VAL_MEASURE_AC_APPARENT_POWER, &app_pow);
			      	SetCtrlVal(panelMeas, PANEL_MEAS_B_app_pow, app_pow);
			   	}
			   	if( checkC == 1 ){
			      	ciCs_Measure (ciCs, "PHASE3", 10000, CICS_VAL_MEASURE_AC_APPARENT_POWER, &app_pow);
				  	SetCtrlVal(panelMeas, PANEL_MEAS_C_app_pow, app_pow);
			   	}
			}
			
			if( meas_peak_curr == 1 ){
			   	if( checkA == 1 ){
					ciCs_Measure (ciCs, "PHASE1", 10000, CICS_VAL_MEASURE_VOLTAGE_MAXIMUM, &peak_curr);
			      	SetCtrlVal(panelMeas, PANEL_MEAS_A_peak_curr, peak_curr);
			   	}
			  	if( checkB == 1 ){
			        ciCs_Measure (ciCs, "PHASE2", 10000, CICS_VAL_MEASURE_VOLTAGE_MAXIMUM, &peak_curr);
			      	SetCtrlVal(panelMeas, PANEL_MEAS_B_peak_curr, peak_curr);
			   	}
			   	if( checkC == 1 ){
			      	ciCs_Measure (ciCs, "PHASE3", 10000, CICS_VAL_MEASURE_VOLTAGE_MAXIMUM, &peak_curr);
			      	SetCtrlVal(panelMeas, PANEL_MEAS_C_peak_curr, peak_curr);
			   	}
			}
			
			if( meas_crest_fact == 1 ){
			  	if( checkA == 1 ){
					ciCs_Measure (ciCs, "PHASE1", 10000, CICS_VAL_MEASURE_VOLTAGE_CRESTFACTOR, &crest_fact);
			      	SetCtrlVal(panelMeas, PANEL_MEAS_A_crest_fact, crest_fact);
			   	}
			   	if( checkB == 1 ){
			      	ciCs_Measure (ciCs, "PHASE2", 10000, CICS_VAL_MEASURE_VOLTAGE_CRESTFACTOR, &crest_fact);
			      	SetCtrlVal(panelMeas, PANEL_MEAS_B_crest_fact, crest_fact);
			    }
			   if( checkC == 1 ){
			      	ciCs_Measure (ciCs, "PHASE3", 10000, CICS_VAL_MEASURE_VOLTAGE_CRESTFACTOR, &crest_fact);
			      	SetCtrlVal(panelMeas, PANEL_MEAS_C_crest_fact, crest_fact);
			   }
			}
			
			SetCtrlAttribute(panelMeas, PANEL_MEAS_meas_button, ATTR_DIMMED, 0);
			break;
			
		case EVENT_RIGHT_CLICK:
		    MessagePopup("Cs Series Controller","Measure all selected options.");
			break;
	}
	return 0;
}

int CVICALLBACK close_meas (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
			DiscardPanel (panelMeas);
		    
			break;
		case EVENT_RIGHT_CLICK:
		    MessagePopup("Cs Series Controller","Quit Measure program.");
	}
	return 0;
}

/********************************** HARMONIC CONTROL **********************************/
void CVICALLBACK Harmonic_panel (int menuBar, int menuItem, void *callbackData,
		int panel)
{
  	panelHarm = LoadPanel (0, "ciCs_example.uir", PANEL_HARM);
   	DisplayPanel (panelHarm); 
   	display_waveform();				
}

void display_waveform(void)
{
  	ViInt32 *maxNumWave;
  	ViInt32 *numPoint;
  	ViInt32 *minNumWave;
  	ViInt32 *maxWaveSize;
  	
  	SetCtrlVal(panelHarm, PANEL_HARM_steady_message, "Wait for collecting waveform list...");  
	
	/* Get waveform list */
   	ClearListCtrl (panelHarm, PANEL_HARM_Waveform_box);  
   	ciCs_QueryDefinedWaveforms (ciCs, waveform);        
   	
   	/* Extract waveform name */
   	i = 0;
   	n = strlen(waveform);
   	index = 1;    /* remove the first begining " quote */
   	while( n > 0 ){
      	y = Scan(waveform, "%s[i*]>%s[t34]", index, wave);  
      	y = strlen(wave) + 1;
      	index = y + index;
      	n = n - y;
      	if( n > 0 ){
         	if( strcmp(wave, ",") == 0 ){
        	}
        	else{
        		if( index > 32 ) 
        			InsertListItem (panelHarm, PANEL_HARM_Waveform_box, -1, wave, 0);  
        	}
       	}
      	i = i++;
   	}
    GetNumListItems (panelHarm, PANEL_HARM_Waveform_box, &maxitems); 
    
    SetCtrlVal(panelHarm, PANEL_HARM_steady_message, "Ready...");
    SetCtrlVal(panelHarm, PANEL_HARM_waveform_name, "");
}

void collect_data (ViReal64 data[])
{
   	ViReal64 oddvalue[25];
    ViReal64 evenvalue[25];
    ViReal64 Amp, angle=0;
    ViReal64 pi = 6.141921E-03;
    int j; 
   
    for( j = 0; j < 1024; j++ ){
     	data[j] = 0.00;
    }
          
    GetCtrlVal(panelHarm, PANEL_HARM_odd_harm_0, &oddvalue[0]);
    GetCtrlVal(panelHarm, PANEL_HARM_odd_harm_1, &oddvalue[1]); 
	GetCtrlVal(panelHarm, PANEL_HARM_odd_harm_2, &oddvalue[2]);  
    GetCtrlVal(panelHarm, PANEL_HARM_odd_harm_3, &oddvalue[3]); 
	GetCtrlVal(panelHarm, PANEL_HARM_odd_harm_4, &oddvalue[4]);  	   
	GetCtrlVal(panelHarm, PANEL_HARM_odd_harm_5, &oddvalue[5]); 
	GetCtrlVal(panelHarm, PANEL_HARM_odd_harm_6, &oddvalue[6]);  	   
	GetCtrlVal(panelHarm, PANEL_HARM_odd_harm_7, &oddvalue[7]); 
	GetCtrlVal(panelHarm, PANEL_HARM_odd_harm_8, &oddvalue[8]);  	   
	GetCtrlVal(panelHarm, PANEL_HARM_odd_harm_9, &oddvalue[9]); 
	GetCtrlVal(panelHarm, PANEL_HARM_odd_harm_10, &oddvalue[10]);  	   
	GetCtrlVal(panelHarm, PANEL_HARM_odd_harm_11, &oddvalue[11]); 
	GetCtrlVal(panelHarm, PANEL_HARM_odd_harm_12, &oddvalue[12]);  	   
	GetCtrlVal(panelHarm, PANEL_HARM_odd_harm_13, &oddvalue[13]); 
	GetCtrlVal(panelHarm, PANEL_HARM_odd_harm_14, &oddvalue[14]);  	   
	GetCtrlVal(panelHarm, PANEL_HARM_odd_harm_15, &oddvalue[15]); 
	GetCtrlVal(panelHarm, PANEL_HARM_odd_harm_16, &oddvalue[16]);  	   
	GetCtrlVal(panelHarm, PANEL_HARM_odd_harm_17, &oddvalue[17]); 
	GetCtrlVal(panelHarm, PANEL_HARM_odd_harm_18, &oddvalue[18]);  	   
	GetCtrlVal(panelHarm, PANEL_HARM_odd_harm_19, &oddvalue[19]); 
	GetCtrlVal(panelHarm, PANEL_HARM_odd_harm_20, &oddvalue[20]);  	   
	GetCtrlVal(panelHarm, PANEL_HARM_odd_harm_21, &oddvalue[21]); 
	GetCtrlVal(panelHarm, PANEL_HARM_odd_harm_22, &oddvalue[22]);  	   
	GetCtrlVal(panelHarm, PANEL_HARM_odd_harm_23, &oddvalue[23]);  	   
	GetCtrlVal(panelHarm, PANEL_HARM_odd_harm_24, &oddvalue[24]); 
		   
	for( i = 0; i <= 24; i++ ){
	  	Amp = oddvalue[i] / 100;
	   	if( Amp != 0 ){
		  	for( j = 0; j < 1024; j++ ){
		     	angle = j * pi * (((i+1) * 2) - 1);
		     	data[j] = data[j] + Amp * sin(angle);
		    }
		}
	}
		   
	GetCtrlVal(panelHarm, PANEL_HARM_even_harm_0, &evenvalue[0]);
	GetCtrlVal(panelHarm, PANEL_HARM_even_harm_1, &evenvalue[1]); 
	GetCtrlVal(panelHarm, PANEL_HARM_even_harm_2, &evenvalue[2]);  
	GetCtrlVal(panelHarm, PANEL_HARM_even_harm_3, &evenvalue[3]); 
	GetCtrlVal(panelHarm, PANEL_HARM_even_harm_4, &evenvalue[4]);  	   
	GetCtrlVal(panelHarm, PANEL_HARM_even_harm_5, &evenvalue[5]); 
	GetCtrlVal(panelHarm, PANEL_HARM_even_harm_6, &evenvalue[6]);  	   
	GetCtrlVal(panelHarm, PANEL_HARM_even_harm_7, &evenvalue[7]); 
	GetCtrlVal(panelHarm, PANEL_HARM_even_harm_8, &evenvalue[8]);  	   
	GetCtrlVal(panelHarm, PANEL_HARM_even_harm_9, &evenvalue[9]); 
	GetCtrlVal(panelHarm, PANEL_HARM_even_harm_10, &evenvalue[10]);  	   
	GetCtrlVal(panelHarm, PANEL_HARM_even_harm_11, &evenvalue[11]); 
	GetCtrlVal(panelHarm, PANEL_HARM_even_harm_12, &evenvalue[12]);  	   
	GetCtrlVal(panelHarm, PANEL_HARM_even_harm_13, &evenvalue[13]); 
	GetCtrlVal(panelHarm, PANEL_HARM_even_harm_14, &evenvalue[14]);  	   
	GetCtrlVal(panelHarm, PANEL_HARM_even_harm_15, &evenvalue[15]); 
	GetCtrlVal(panelHarm, PANEL_HARM_even_harm_16, &evenvalue[16]);  	   
	GetCtrlVal(panelHarm, PANEL_HARM_even_harm_17, &evenvalue[17]); 
	GetCtrlVal(panelHarm, PANEL_HARM_even_harm_18, &evenvalue[18]);  	   
	GetCtrlVal(panelHarm, PANEL_HARM_even_harm_19, &evenvalue[19]); 
	GetCtrlVal(panelHarm, PANEL_HARM_even_harm_20, &evenvalue[20]);  	   
	GetCtrlVal(panelHarm, PANEL_HARM_even_harm_21, &evenvalue[21]); 
	GetCtrlVal(panelHarm, PANEL_HARM_even_harm_22, &evenvalue[22]);  	   
	GetCtrlVal(panelHarm, PANEL_HARM_even_harm_23, &evenvalue[23]);  	   
	GetCtrlVal(panelHarm, PANEL_HARM_even_harm_24, &evenvalue[24]); 
	  
	for( i = 0; i <= 24; i++ ){
	  	Amp = evenvalue[i] / 100;
	   	if( Amp != 0 ){
		  	for( j = 0; j < 1024; j++ ){
		     	angle = j * pi * ((i+1) * 2);
		     	data[j] = data[j] + Amp * sin(angle);
		    }
		}
	}  
}

int CVICALLBACK harmonic_download (int panel, int control, int event, void *callbackData,
		int eventData1, int eventData2)
{
    char wave_name[12];
    
	switch (event) {
		case EVENT_COMMIT: 
	    	SetCtrlVal(panelHarm, PANEL_HARM_steady_message, "Wait for downloading waveform to AC Source...");     
		   	collect_data(data);	
		   	GetCtrlVal( panelHarm, PANEL_HARM_waveform_name, wave_name ); 
		   	StringUpperCase( wave_name );
		   	ciCs_WriteArbWaveform (ciCs, wave_name, 1024, data);
		   	
			display_waveform();
			break;
		case EVENT_LOST_FOCUS:
			break;
		case EVENT_CLOSE:
			DiscardPanel (panelHarm);
	   
		break;
	}
	return 0;
}

int CVICALLBACK harmonic_preview (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
		    collect_data(data);
			DeleteGraphPlot( panelHarm, PANEL_HARM_GRAPH, -1, 0 );

            PlotY( panelHarm, PANEL_HARM_GRAPH, data, 1024, VAL_DOUBLE, VAL_THIN_LINE,
               VAL_EMPTY_SQUARE, VAL_SOLID, 1, VAL_RED );
			break;
	}
	return 0;
}

int CVICALLBACK harmonic_delete (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	char label[12];
	
	switch (event) {
		case EVENT_COMMIT:
			GetNumCheckedItems (panelHarm, PANEL_HARM_Waveform_box, &numitems); 
			if( numitems > 1 ){
			   	MessagePopup ("Cs Series Controller", "Choose one waveform only.");   
			   	break;
			}
			GetNumListItems( panelHarm, PANEL_HARM_Waveform_box, &maxitems ); 
			status = ConfirmPopup ("Cs Series Controller", "Are you sure you want to delete all waveforms from AC Source memory?");
	      	if( status ){
				ciCs_ClearAllArbWaveforms (ciCs);
	        }
			display_waveform();  
			break;
	}
	return 0;
}

int CVICALLBACK deleteWave (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	char waveform[20];
	
	switch (event) {
		case EVENT_COMMIT:
			GetNumCheckedItems (panelHarm, PANEL_HARM_Waveform_box, &numitems); 
			if( numitems > 1 ){
			   	MessagePopup ("Cs Series Controller", "Choose one waveform only.");   
			   	break;
			}
			GetNumListItems( panelHarm, PANEL_HARM_Waveform_box, &maxitems ); 
			for( i = 0; i < maxitems; i++ ){
			   	IsListItemChecked (panelHarm, PANEL_HARM_Waveform_box, i, &checked);
			   	if( checked ){
			      	status = ConfirmPopup ("Cs Series Controller", "Are you sure you want to delete this waveform from AC Source memory?");
			      	if( status ){
			         	GetLabelFromIndex (panelHarm, PANEL_HARM_Waveform_box, i, waveform); 
			         	
						ciCs_ClearArbWaveforms (ciCs, waveform);
			        }
			   	}
			}
		    display_waveform();  
			break;
	}
	return 0;
}

int CVICALLBACK harmonic_clear (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	int j;
	
	switch (event) {
		case EVENT_COMMIT:
			status = ConfirmPopup ("Cs Series Controller", "This operation will earse all harmonic table. Are you sure you want to continue?");
			if( status ){
              	DeleteGraphPlot (panelHarm, PANEL_HARM_GRAPH, -1, 1);
                
                SetCtrlVal(panelHarm, PANEL_HARM_odd_harm_1, 0.00); 
                SetCtrlVal(panelHarm, PANEL_HARM_odd_harm_2, 0.00); 
                SetCtrlVal(panelHarm, PANEL_HARM_odd_harm_3, 0.00); 
                SetCtrlVal(panelHarm, PANEL_HARM_odd_harm_4, 0.00); 
                SetCtrlVal(panelHarm, PANEL_HARM_odd_harm_5, 0.00); 
                SetCtrlVal(panelHarm, PANEL_HARM_odd_harm_6, 0.00); 
                SetCtrlVal(panelHarm, PANEL_HARM_odd_harm_7, 0.00); 
                SetCtrlVal(panelHarm, PANEL_HARM_odd_harm_8, 0.00); 			        
                SetCtrlVal(panelHarm, PANEL_HARM_odd_harm_9, 0.00); 
                SetCtrlVal(panelHarm, PANEL_HARM_odd_harm_10, 0.00); 
                SetCtrlVal(panelHarm, PANEL_HARM_odd_harm_11, 0.00); 
                SetCtrlVal(panelHarm, PANEL_HARM_odd_harm_12, 0.00); 
                SetCtrlVal(panelHarm, PANEL_HARM_odd_harm_13, 0.00); 
                SetCtrlVal(panelHarm, PANEL_HARM_odd_harm_14, 0.00); 
                SetCtrlVal(panelHarm, PANEL_HARM_odd_harm_15, 0.00); 
                SetCtrlVal(panelHarm, PANEL_HARM_odd_harm_16, 0.00); 
                SetCtrlVal(panelHarm, PANEL_HARM_odd_harm_17, 0.00); 
                SetCtrlVal(panelHarm, PANEL_HARM_odd_harm_18, 0.00); 
                SetCtrlVal(panelHarm, PANEL_HARM_odd_harm_19, 0.00); 			        
                SetCtrlVal(panelHarm, PANEL_HARM_odd_harm_20, 0.00); 
                SetCtrlVal(panelHarm, PANEL_HARM_odd_harm_21, 0.00); 
                SetCtrlVal(panelHarm, PANEL_HARM_odd_harm_22, 0.00); 
                SetCtrlVal(panelHarm, PANEL_HARM_odd_harm_23, 0.00); 
                SetCtrlVal(panelHarm, PANEL_HARM_odd_harm_24, 0.00);
                  
                SetCtrlVal(panelHarm, PANEL_HARM_even_harm_0, 0.00); 
                SetCtrlVal(panelHarm, PANEL_HARM_even_harm_1, 0.00); 
                SetCtrlVal(panelHarm, PANEL_HARM_even_harm_2, 0.00); 
                SetCtrlVal(panelHarm, PANEL_HARM_even_harm_3, 0.00); 
                SetCtrlVal(panelHarm, PANEL_HARM_even_harm_4, 0.00); 
                SetCtrlVal(panelHarm, PANEL_HARM_even_harm_5, 0.00); 
                SetCtrlVal(panelHarm, PANEL_HARM_even_harm_6, 0.00); 
                SetCtrlVal(panelHarm, PANEL_HARM_even_harm_7, 0.00); 
                SetCtrlVal(panelHarm, PANEL_HARM_even_harm_8, 0.00); 
                SetCtrlVal(panelHarm, PANEL_HARM_even_harm_9, 0.00); 
                SetCtrlVal(panelHarm, PANEL_HARM_even_harm_10, 0.00); 
                SetCtrlVal(panelHarm, PANEL_HARM_even_harm_11, 0.00); 
                SetCtrlVal(panelHarm, PANEL_HARM_even_harm_12, 0.00); 
                SetCtrlVal(panelHarm, PANEL_HARM_even_harm_13, 0.00); 
                SetCtrlVal(panelHarm, PANEL_HARM_even_harm_14, 0.00); 
                SetCtrlVal(panelHarm, PANEL_HARM_even_harm_15, 0.00); 
                SetCtrlVal(panelHarm, PANEL_HARM_even_harm_16, 0.00); 
                SetCtrlVal(panelHarm, PANEL_HARM_even_harm_17, 0.00); 
                SetCtrlVal(panelHarm, PANEL_HARM_even_harm_18, 0.00); 
                SetCtrlVal(panelHarm, PANEL_HARM_even_harm_19, 0.00); 
                SetCtrlVal(panelHarm, PANEL_HARM_even_harm_20, 0.00); 
                SetCtrlVal(panelHarm, PANEL_HARM_even_harm_21, 0.00); 
                SetCtrlVal(panelHarm, PANEL_HARM_even_harm_22, 0.00); 
                SetCtrlVal(panelHarm, PANEL_HARM_even_harm_23, 0.00); 
                SetCtrlVal(panelHarm, PANEL_HARM_even_harm_24, 0.00); 
			}
			
			for( j = 0; j < 1024; j++ ){
		     	data[j] = 0.00;
		    }
		    
			break;
	}
	return 0;
}

int CVICALLBACK waveform_list(int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	char label[12];
	
	switch (event) {
		case EVENT_COMMIT:
            GetNumCheckedItems (panelHarm, PANEL_HARM_Waveform_box, &numitems); 
			if( numitems > 1 ){
			   	MessagePopup ("Cs Series Controller", "Choose one waveform only.");   
			   	display_waveform();
			   	break;
			}
			GetNumListItems (panelHarm, PANEL_HARM_Waveform_box, &maxitems); 
			for( i = 0; i < maxitems; i++ ){
			   	IsListItemChecked (panelHarm, PANEL_HARM_Waveform_box, i, &checked);
			   	if( checked ){
			      	GetLabelFromIndex (panelHarm, PANEL_HARM_Waveform_box, i, label); 
			      	SetCtrlVal(panelHarm, PANEL_HARM_waveform_name, label);
			   	}
			}
	   		break;
	}
	return 0;
}

int CVICALLBACK close_harm (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
			DiscardPanel (panelHarm);
			break;
		case EVENT_RIGHT_CLICK:
		    MessagePopup("Cs Series Controller","Quit Harmonic program.");
	}
	return 0;
}

/*****************************************************************************
 * Function: BuildErrorString   
 * Purpose:  This function builds a string that describes the error that the 
 *           sample program encounters using the error information functions 
 *           that the instrument driver exports.
 *****************************************************************************/
void BuildErrorString (ViSession ciCs, ViStatus error, ViString errStr)
{

    ViChar   tempStr[256], errElabStr[256];
    ViChar   *p = errStr;
    ViStatus primaryErr, secondaryErr;
            
    p += sprintf (p, "Driver Status:  (Hex 0x%x)", error);
                                
        /* Get description of the returned status code */
    ciCs_error_message (ciCs, error, tempStr);
    p += sprintf (p, " %s\n\n", tempStr);
        
        /* Get any additional error information */
    errElabStr[0] = 0;
    ciCs_GetErrorInfo (ciCs, &primaryErr, &secondaryErr, errElabStr);
    
    if (primaryErr == error)
        {
        if (secondaryErr != VI_SUCCESS)
            {
                p += sprintf (p, "Secondary Error:  (Hex 0x%x)", secondaryErr);
                                
                    /* Get description of the returned status code */
                ciCs_error_message (ciCs, secondaryErr, tempStr);
                p += sprintf (p, " %s\n\n", tempStr);
            }
        if (errElabStr[0] != 0)
            sprintf (p, "Elaboration:  %s", errElabStr);
        }
}

//========= End of Sample Program ==============//

	  


